#include <stdio.h>
#include "system.h"
#include "string.h"
#include "altera_avalon_pio_regs.h"
#include <unistd.h>
#include "io.h"
#include "altera_avalon_jtag_uart_regs.h"


#define NUMBER_OF_MS_PER_HOUR 3600000
#define NUMBER_OF_MS_PER_MINUTE 60000
#define NUMBER_OF_MS_PER_SECOND 1000
#define NUMBER_OF_LINKS 4
#define NUMBER_OF_QUADS 1
#define NUMBER_OF_REFCLKS 3
#define NUMBER_OF_DATARATES 4
#define MULTIPLIER 65536 // 2^16
#define DEBUG 0

#define VOD 0
#define PRE_TAP 1
#define POST_TAP1 2
#define POST_TAP2 3
#define EQ_GAIN 16 
#define EQ_CTRL 17 
#define PRECDR 32 
#define POSTCDR 33 


#define RX_5400MBPS_4  0x5400   

#define RX_2700MBPS_4  0x5A00   

#define RX_1620MBPS_4  0x9600  

#define RX_2970MBPS_4  0x5600  


#define PLL_5400MBPS_4 0x7401   
#define PLL_5400MBPS_6 0x0D60
#define PLL_5400MBPS_7 0x8006

#define PLL_2700MBPS_4 0x5801 
#define PLL_2700MBPS_6 0x0D60
#define PLL_2700MBPS_7 0x8006

#define PLL_1620MBPS_4 0x9801  
#define PLL_1620MBPS_6 0x0D68
#define PLL_1620MBPS_7 0x8006

#define PLL_2970MBPS_4 0x3801  
#define PLL_2970MBPS_6 0x0560
#define PLL_2970MBPS_7 0x8006

#define RX_5400MBPS_REFCLK 0x0000
#define RX_2700MBPS_REFCLK 0x0000
#define RX_1620MBPS_REFCLK 0x0000
#define RX_2970MBPS_REFCLK 0x0001

#define TX_5400MBPS_REFCLK 0x0000
#define TX_2700MBPS_REFCLK 0x0000
#define TX_1620MBPS_REFCLK 0x0000
#define TX_2970MBPS_REFCLK 0x0001


#define VOD_200MV 10
#define VOD_400MV 20
#define VOD_600MV 30
#define VOD_700MV 35
#define VOD_800MV 40
#define VOD_900MV 45
#define VOD_1000MV 50
#define VOD_1200MV 60

#define PRE_TAP_15N 1
#define PRE_TAP_14N 2
#define PRE_TAP_13N 3
#define PRE_TAP_12N 4
#define PRE_TAP_11N 5
#define PRE_TAP_10N 6
#define PRE_TAP_9N 7
#define PRE_TAP_8N 8
#define PRE_TAP_7N 9
#define PRE_TAP_6N 10
#define PRE_TAP_5N 11
#define PRE_TAP_4N 12
#define PRE_TAP_3N 13
#define PRE_TAP_2N 14
#define PRE_TAP_1N 15
#define PRE_TAP_0  16 //0 or 16 for this setting
#define PRE_TAP_1  17
#define PRE_TAP_2  18
#define PRE_TAP_3  19
#define PRE_TAP_4  20
#define PRE_TAP_5  21
#define PRE_TAP_6  22
#define PRE_TAP_7  23
#define PRE_TAP_8  24
#define PRE_TAP_9  25
#define PRE_TAP_10  26
#define PRE_TAP_11  27
#define PRE_TAP_12  28
#define PRE_TAP_13  29
#define PRE_TAP_14  30
#define PRE_TAP_15  31

#define POST_TAP1_0 0
#define POST_TAP1_1 1
#define POST_TAP1_2 2
#define POST_TAP1_3 3
#define POST_TAP1_4 4
#define POST_TAP1_5 5
#define POST_TAP1_6 6
#define POST_TAP1_7 7
#define POST_TAP1_8 8
#define POST_TAP1_9 9
#define POST_TAP1_10 10
#define POST_TAP1_11 11
#define POST_TAP1_12 12
#define POST_TAP1_13 13
#define POST_TAP1_14 14
#define POST_TAP1_15 15
#define POST_TAP1_16 16
#define POST_TAP1_17 17
#define POST_TAP1_18 18
#define POST_TAP1_19 19
#define POST_TAP1_20 20
#define POST_TAP1_21 21
#define POST_TAP1_22 22
#define POST_TAP1_23 23
#define POST_TAP1_24 24
#define POST_TAP1_25 25
#define POST_TAP1_26 26
#define POST_TAP1_27 27
#define POST_TAP1_28 28
#define POST_TAP1_29 29
#define POST_TAP1_30 30
#define POST_TAP1_31 31


#define POST_TAP2_15N 1
#define POST_TAP2_14N 2
#define POST_TAP2_13N 3
#define POST_TAP2_12N 4
#define POST_TAP2_11N 5
#define POST_TAP2_10N 6
#define POST_TAP2_9N 7
#define POST_TAP2_8N 8
#define POST_TAP2_7N 9
#define POST_TAP2_6N 10
#define POST_TAP2_5N 11
#define POST_TAP2_4N 12
#define POST_TAP2_3N 13
#define POST_TAP2_2N 14
#define POST_TAP2_1N 15
#define POST_TAP2_0  16 //0 or 16 for this setting
#define POST_TAP2_1  17
#define POST_TAP2_2  18
#define POST_TAP2_3  19
#define POST_TAP2_4  20
#define POST_TAP2_5  21
#define POST_TAP2_6  22
#define POST_TAP2_7  23
#define POST_TAP2_8  24
#define POST_TAP2_9  25
#define POST_TAP2_10  26
#define POST_TAP2_11  27
#define POST_TAP2_12  28
#define POST_TAP2_13  29
#define POST_TAP2_14  30
#define POST_TAP2_15  31

#define EQ_CTRL_0 0
#define EQ_CTRL_1 1
#define EQ_CTRL_2 2
#define EQ_CTRL_3 3
#define EQ_CTRL_4 4
#define EQ_CTRL_5 5
#define EQ_CTRL_6 6
#define EQ_CTRL_7 7
#define EQ_CTRL_8 8
#define EQ_CTRL_9 9
#define EQ_CTRL_10 10
#define EQ_CTRL_11 11
#define EQ_CTRL_12 12
#define EQ_CTRL_13 13
#define EQ_CTRL_14 14
#define EQ_CTRL_15 15

#define EQ_GAIN_0 0
#define EQ_GAIN_1 1
#define EQ_GAIN_2 2
#define EQ_GAIN_3 3
#define EQ_GAIN_4 4
#define EQ_GAIN_5 5
#define EQ_GAIN_6 6
#define EQ_GAIN_7 7

#define PRBS_7 0
#define PRBS_23 1
#define PRBS_31 2
#define PRBS_15 3

#define RATE_5400MBPS 0
#define RATE_2700MBPS 1
#define RATE_1620MBPS 2
#define RATE_2970MBPS 3

/* Functions */

/*-----------------------------------------------
 * input_char()
 * 
 * Gets a single input char for menu prompt
 * 
 *   ******** Added by Guy for Menu inputs
 *   ******** Changed by Maurizio for Nios II SDK Shell support
 * 
 * ----------------------------------------------*/
char input_char(void)
 {
 char in_char, rx_char;
 
   rx_char = 0;
   while (1)
   {
     in_char = getchar();
     if (in_char != '\r' && in_char != '\n')
         rx_char = in_char;
     else if (in_char == '\n')
         break;
   };
 return (rx_char);
 }

int input_number(void)
 {
 char in_char,LSB;
 int temp, number;
 
   LSB = 0;
   temp = 0;
   number = 0;
   while (1)
   {
     in_char = getchar();
     if (in_char != '\r' && in_char != '\n')
        {
         //rx_char = in_char;        
         LSB = in_char;
        }
     else if (in_char == '\n')
         break;
          
            switch (LSB)
            {
            case '0': temp = 0; break;
            case '1': temp = 1; break;
            case '2': temp = 2; break;
            case '3': temp = 3; break;
            case '4': temp = 4; break;
            case '5': temp = 5; break;
            case '6': temp = 6; break;
            case '7': temp = 7; break;
            case '8': temp = 8; break;
            case '9': temp = 9; break;
            case 'A': 
            case 'a': temp = 0xa; break;
            case 'B': 
            case 'b': temp = 0xb; break;
            case 'C': 
            case 'c': temp = 0xc; break;
            case 'D': 
            case 'd': temp = 0xd; break;
            case 'E': 
            case 'e': temp = 0xe; break;
            case 'F': 
            case 'f': temp = 0xf; break;
            default : break;
            }
         
    
          number  = temp;         
   };
 return (number);
 } 
 
int input_byte(void)
 {
 char in_char, MSB,LSB;
 int temp, High_nibble, Low_nibble, Byte;
 
   MSB = 0;
   LSB = 0;
   temp = 0;
   Byte = 0;
   while (1)
   {
     in_char = getchar();
     if (in_char != '\r' && in_char != '\n')
        {
         //rx_char = in_char;
         MSB = LSB;         
         LSB = in_char;
        }
     else if (in_char == '\n')
         break;
     switch (MSB)
            {
            case '0': temp = 0; break;
            case '1': temp = 1; break;
            case '2': temp = 2; break;
            case '3': temp = 3; break;
            case '4': temp = 4; break;
            case '5': temp = 5; break;
            case '6': temp = 6; break;
            case '7': temp = 7; break;
            case '8': temp = 8; break;
            case '9': temp = 9; break;
            case 'A': 
            case 'a': temp = 0xa; break;
            case 'B': 
            case 'b': temp = 0xb; break;
            case 'C': 
            case 'c': temp = 0xc; break;
            case 'D': 
            case 'd': temp = 0xd; break;
            case 'E': 
            case 'e': temp = 0xe; break;
            case 'F': 
            case 'f': temp = 0xf; break;
            default : break;
            }
          
          High_nibble = temp;
          
            switch (LSB)
            {
            case '0': temp = 0; break;
            case '1': temp = 1; break;
            case '2': temp = 2; break;
            case '3': temp = 3; break;
            case '4': temp = 4; break;
            case '5': temp = 5; break;
            case '6': temp = 6; break;
            case '7': temp = 7; break;
            case '8': temp = 8; break;
            case '9': temp = 9; break;
            case 'A': 
            case 'a': temp = 0xa; break;
            case 'B': 
            case 'b': temp = 0xb; break;
            case 'C': 
            case 'c': temp = 0xc; break;
            case 'D': 
            case 'd': temp = 0xd; break;
            case 'E': 
            case 'e': temp = 0xe; break;
            case 'F': 
            case 'f': temp = 0xf; break;
            default : break;
            }
          
          Low_nibble = temp;  
    
          Byte = (High_nibble << 4) | Low_nibble;         
   };
 return (Byte);
 } 


 
int input_word(void)
 {
 char in_char, N3,N2,N1,N0;
 int temp, N3_int, N2_int,N1_int,N0_int, Word;
 
   N3 = 0;
   N2 = 0;
   N1 = 0;
   N0 = 0;         
   temp = 0;
   Word = 0;
   while (1)
   {
     in_char = getchar();
     if (in_char != '\r' && in_char != '\n')
        {
         //rx_char = in_char;
         N3 = N2;
         N2 = N1;
         N1 = N0;
         N0 = in_char;
        }
     else if (in_char == '\n')
         break;
     switch (N3)
            {
            case '0': temp = 0; break;
            case '1': temp = 1; break;
            case '2': temp = 2; break;
            case '3': temp = 3; break;
            case '4': temp = 4; break;
            case '5': temp = 5; break;
            case '6': temp = 6; break;
            case '7': temp = 7; break;
            case '8': temp = 8; break;
            case '9': temp = 9; break;
            case 'A': 
            case 'a': temp = 0xa; break;
            case 'B': 
            case 'b': temp = 0xb; break;
            case 'C': 
            case 'c': temp = 0xc; break;
            case 'D': 
            case 'd': temp = 0xd; break;
            case 'E': 
            case 'e': temp = 0xe; break;
            case 'F': 
            case 'f': temp = 0xf; break;
            default : break;
            }
          
          N3_int = temp;
          
         switch (N2)
            {
            case '0': temp = 0; break;
            case '1': temp = 1; break;
            case '2': temp = 2; break;
            case '3': temp = 3; break;
            case '4': temp = 4; break;
            case '5': temp = 5; break;
            case '6': temp = 6; break;
            case '7': temp = 7; break;
            case '8': temp = 8; break;
            case '9': temp = 9; break;
            case 'A': 
            case 'a': temp = 0xa; break;
            case 'B': 
            case 'b': temp = 0xb; break;
            case 'C': 
            case 'c': temp = 0xc; break;
            case 'D': 
            case 'd': temp = 0xd; break;
            case 'E': 
            case 'e': temp = 0xe; break;
            case 'F': 
            case 'f': temp = 0xf; break;
            default : break;
            }
          
          N2_int = temp;  

         switch (N1)
            {
            case '0': temp = 0; break;
            case '1': temp = 1; break;
            case '2': temp = 2; break;
            case '3': temp = 3; break;
            case '4': temp = 4; break;
            case '5': temp = 5; break;
            case '6': temp = 6; break;
            case '7': temp = 7; break;
            case '8': temp = 8; break;
            case '9': temp = 9; break;
            case 'A': 
            case 'a': temp = 0xa; break;
            case 'B': 
            case 'b': temp = 0xb; break;
            case 'C': 
            case 'c': temp = 0xc; break;
            case 'D': 
            case 'd': temp = 0xd; break;
            case 'E': 
            case 'e': temp = 0xe; break;
            case 'F': 
            case 'f': temp = 0xf; break;
            default : break;
            }
          
          N1_int = temp;  

         switch (N0)
            {
            case '0': temp = 0; break;
            case '1': temp = 1; break;
            case '2': temp = 2; break;
            case '3': temp = 3; break;
            case '4': temp = 4; break;
            case '5': temp = 5; break;
            case '6': temp = 6; break;
            case '7': temp = 7; break;
            case '8': temp = 8; break;
            case '9': temp = 9; break;
            case 'A': 
            case 'a': temp = 0xa; break;
            case 'B': 
            case 'b': temp = 0xb; break;
            case 'C': 
            case 'c': temp = 0xc; break;
            case 'D': 
            case 'd': temp = 0xd; break;
            case 'E': 
            case 'e': temp = 0xe; break;
            case 'F': 
            case 'f': temp = 0xf; break;
            default : break;
            }
          
          N0_int = temp;                
          Word = (N3_int << 12) | (N2_int << 8) | (N1_int << 4) | N0_int;         
   };
 return (Word);
 } 
 
 void write_reconfiguration(int logical_channel_address,int tx_rx_word_offset, int reconfig_data)
 {
	 int Control_and_Status;
	 int Error;
	 int Busy;
	 //int readback;	 

	  
	  IOWR(RECONFIG_MGMT_0_BASE,0x008,logical_channel_address);

	  
	  IOWR(RECONFIG_MGMT_0_BASE,0x00B,tx_rx_word_offset);	  
	  
	  IOWR(RECONFIG_MGMT_0_BASE,0x00C,reconfig_data);	
	  
	  IOWR(RECONFIG_MGMT_0_BASE,0x00A,0x0001); // Perform a write operation

    do
    {
     Control_and_Status = IORD(RECONFIG_MGMT_0_BASE,0x00A);
	  Busy = 0x0001 & (Control_and_Status >> 8); 
	  } while (Busy == 1);
		  
	  
	  Control_and_Status = IORD(RECONFIG_MGMT_0_BASE,0x00A);
	  Error = 0x0001 & (Control_and_Status >> 9); 	  
}  


 void write_reconfiguration_EyeQ(int logical_channel_address,int tx_rx_word_offset, int reconfig_data)
 {
	 int Control_and_Status;
	 int Error;
	 int Busy;
	 //int readback;	 

	  
	  IOWR(RECONFIG_MGMT_0_BASE,0x010,logical_channel_address);

	  
	  IOWR(RECONFIG_MGMT_0_BASE,0x013,tx_rx_word_offset);	  
	  
	  IOWR(RECONFIG_MGMT_0_BASE,0x014,reconfig_data);	
	  
	  IOWR(RECONFIG_MGMT_0_BASE,0x012,0x0001); // Perform a write operation

    do
    {
     Control_and_Status = IORD(RECONFIG_MGMT_0_BASE,0x012);
	  Busy = 0x0001 & (Control_and_Status >> 8); 
	  } while (Busy == 1);
		  
	  
	  Control_and_Status = IORD(RECONFIG_MGMT_0_BASE,0x012);
	  Error = 0x0001 & (Control_and_Status >> 9); 	  
}  

int read_reconfiguration_EyeQ(int logical_channel_address,int tx_rx_word_offset)
 {
	 int Control_and_Status;
	 int Error;
	 int Busy;
	 int reconfig_data;
	 //int readback;	 

	  
	  IOWR(RECONFIG_MGMT_0_BASE,0x010,logical_channel_address);

	  
	  IOWR(RECONFIG_MGMT_0_BASE,0x013,tx_rx_word_offset);	  
	
	  
	  IOWR(RECONFIG_MGMT_0_BASE,0x012,0x0002); // Perform a read operation

    do
    {
     Control_and_Status = IORD(RECONFIG_MGMT_0_BASE,0x012);
	  Busy = 0x0001 & (Control_and_Status >> 8); 
	  } while (Busy == 1);
		  
	  Control_and_Status = IORD(RECONFIG_MGMT_0_BASE,0x012);
	  Error = 0x0001 & (Control_and_Status >> 9); 	  
	  
	  reconfig_data = IORD(RECONFIG_MGMT_0_BASE,0x014); 
	 return(reconfig_data);
} 


// void check_busy (void)
// {
//  unsigned long Control_and_Status;
//  int Busy;
// 
//     do
//    {
//	  // Read Busy until it's low
//     Control_and_Status = IORD(RECONFIG_MGMT_0_BASE,0x003A);
//         //Busy = 0x0001 & (Control_and_Status >> 8); 
//		   Busy = ((Control_and_Status & 0x00000100) == 0x00000100);
//         } while (Busy == 1);  	
//}

// void write_reconfiguration_mif_streamer_mode_0(int logical_channel_address)
// {
//	 int Control_and_Status;
//	 int Error;
//	 int Busy;
//	 int readback;
//	 int Physical_Channel_Register;
//
//	  
//	  
//	  // Write logical channel address
//	  IOWR(RECONFIG_MGMT_0_BASE,0x038,logical_channel_address);	
//	 
//	 if (DEBUG == 1) 
//		 {
//	  readback = ((0x0000003F) & IORD(RECONFIG_MGMT_0_BASE,0x0038));
//	  printf("Logical Channel Address read back: 0x%2x\n",readback);
//		 }
//		 
//	 
//	  // Set Streamer to mode 0
//	  IOWR(RECONFIG_MGMT_0_BASE,0x03A,0x0); 
//
//	  
//	  // Set Streamer offset register to MIF Base Address 
//	  IOWR(RECONFIG_MGMT_0_BASE,0x03B,0x0); 	
//
//
//	  // Set data register with the MIF Base address 
//	  IOWR(RECONFIG_MGMT_0_BASE,0x03C,0x0); 
//	  
//	  // Writing all data to the streamer 
//	  IOWR(RECONFIG_MGMT_0_BASE,0x03A,0x1); 	
//	 
//	//check_busy();	
//	  
//	  // Set Streamer module offset for Start MIF Streaming
//	  IOWR(RECONFIG_MGMT_0_BASE,0x03B,0x1); 	
//	   
//	  
//	  // Set data register with 0x3 to setup for streaming and use word adressing
//	  IOWR(RECONFIG_MGMT_0_BASE,0x03C,0x3);
//	   
//	  
//	  // Writing all data to the streamer to start streaming the MIF
//	  IOWR(RECONFIG_MGMT_0_BASE,0x03A,0x1); 	  
//	  
//
//	check_busy();	  
//	  
//	  Control_and_Status = IORD(RECONFIG_MGMT_0_BASE,0x03A);
//	  Error = 0x0001 & (Control_and_Status >> 9); 	  
//	  printf("\nError bit %x\n",Error);		  
//} 

 void write_reconfiguration_refclk(int logical_channel_address, int reconfig_data)
 {
	 int Control_and_Status;
	 int Error;
	 int Busy;
	 //int readback;	 

	  
	  IOWR(RECONFIG_MGMT_0_BASE,0x040,logical_channel_address);	
	  
	  IOWR(RECONFIG_MGMT_0_BASE,0x043,0x0); //Refclk selection Offset  
	  
	  IOWR(RECONFIG_MGMT_0_BASE,0x044,reconfig_data);	
	  
	  IOWR(RECONFIG_MGMT_0_BASE,0x042,0x0001); // Perform a write operation

    do
    {
     Control_and_Status = IORD(RECONFIG_MGMT_0_BASE,0x042);
	  Busy = 0x0001 & (Control_and_Status >> 8); 
	  } while (Busy == 1);	  
	  
	  Control_and_Status = IORD(RECONFIG_MGMT_0_BASE,0x042);
	  Error = 0x0001 & (Control_and_Status >> 9); 	  
} 

void write_reconfiguration_direct_register_mode_1(int logical_channel_address,int offset, int reconfig_data)
 {
	 int Control_and_Status;
	 int Error;
	 int Busy;
	 //int readback;	 

	  
	  IOWR(RECONFIG_MGMT_0_BASE,0x038,logical_channel_address);	 // Set logical channel address
	 
	  IOWR(RECONFIG_MGMT_0_BASE,0x03A,0x0004);	//Set streamer mode to 1
	  
	  IOWR(RECONFIG_MGMT_0_BASE,0x03B,offset); // Set offset address	  
	  
	  IOWR(RECONFIG_MGMT_0_BASE,0x03C,reconfig_data);	// Set offset data
	  
	  IOWR(RECONFIG_MGMT_0_BASE,0x03A,0x0005); // Perform a write operation

    do
    {
     Control_and_Status = IORD(RECONFIG_MGMT_0_BASE,0x03A);
	  Busy = 0x0001 & (Control_and_Status >> 8); 
	  } while (Busy == 1);		  
	  
	  Control_and_Status = IORD(RECONFIG_MGMT_0_BASE,0x03A);
	  Error = 0x0001 & (Control_and_Status >> 9); 	  
}  

int read_reconfiguration_direct_register_mode_1(int logical_channel_address,int offset)
 {
	 int Control_and_Status;
	 int Error;
	 int Busy;
	 int read_data;
	 //int readback;	 

	  
	  IOWR(RECONFIG_MGMT_0_BASE,0x038,logical_channel_address);	 // Set logical channel address
	 
	  IOWR(RECONFIG_MGMT_0_BASE,0x03A,0x0004);	//Set streamer mode to 1
	  
	  IOWR(RECONFIG_MGMT_0_BASE,0x03B,offset); // Set offset address	  
	  
	  
	  IOWR(RECONFIG_MGMT_0_BASE,0x03A,0x0006); // Perform a read operation

    do
    {
     Control_and_Status = IORD(RECONFIG_MGMT_0_BASE,0x03A);
	  Busy = 0x0001 & (Control_and_Status >> 8); 
	  } while (Busy == 1);		  
	  
	  Control_and_Status = IORD(RECONFIG_MGMT_0_BASE,0x03A);
	  Error = 0x0001 & (Control_and_Status >> 9); 
	  
	  read_data = IORD(RECONFIG_MGMT_0_BASE,0x03C);
	  printf("\nread_data for logical channel %d at offset %x : %x ",logical_channel_address,offset,read_data);
	 
	  return(read_data);	  
	  
}  

int Read_Channel_Reg( int SelectedChannel)
 {  
int temp;
	switch(SelectedChannel)
	  {
	  case 0 :       
		temp = IORD_ALTERA_AVALON_PIO_DATA(CHANNEL0_REG_BASE);         		   
        break;     
      case 1:           		
        temp = IORD_ALTERA_AVALON_PIO_DATA(CHANNEL1_REG_BASE);   
        break;       
      case 2:
        temp = IORD_ALTERA_AVALON_PIO_DATA(CHANNEL2_REG_BASE);   
        break;         
      case 3:
        temp = IORD_ALTERA_AVALON_PIO_DATA(CHANNEL3_REG_BASE);               
        break;                             
      default: 
		temp = 0;
      }    
 return (temp);      
}

int Read_ErrorCount_L_Reg( int SelectedChannel)
 {  
int temp;
	switch(SelectedChannel)
	  {
	  case 0 :       
		  temp = IORD_ALTERA_AVALON_PIO_DATA(ERRORCOUNT_L_0_BASE);          		   
        break;     
      case 1:           		
        temp = IORD_ALTERA_AVALON_PIO_DATA(ERRORCOUNT_L_1_BASE);   
        break;       
      case 2:
        temp = IORD_ALTERA_AVALON_PIO_DATA(ERRORCOUNT_L_2_BASE);    
        break;         
      case 3:
        temp = IORD_ALTERA_AVALON_PIO_DATA(ERRORCOUNT_L_3_BASE);        
        break;                                    
      default: 
		temp = 0;
      }    
 return (temp);      
}

int Read_ErrorCount_H_Reg( int SelectedChannel)
 {  
int temp;
	switch(SelectedChannel)
	  {
	  case 0 :       
		  temp = IORD_ALTERA_AVALON_PIO_DATA(ERRORCOUNT_H_0_BASE);          		   
        break;     
      case 1:           		
        temp = IORD_ALTERA_AVALON_PIO_DATA(ERRORCOUNT_H_1_BASE);   
        break;       
      case 2:
        temp = IORD_ALTERA_AVALON_PIO_DATA(ERRORCOUNT_H_2_BASE);    
        break;         
      case 3:
        temp = IORD_ALTERA_AVALON_PIO_DATA(ERRORCOUNT_H_3_BASE);        
        break;                                    
      default: 
		temp = 0;
      }    
 return (temp);      
}



int main()
{ 
	char rx_char;
  int Control_Reg[NUMBER_OF_QUADS];
  int Control2_Reg[NUMBER_OF_QUADS];
  int Bitrate[NUMBER_OF_DATARATES];		
  float Bitrate_temp[NUMBER_OF_DATARATES];
  int Refclockmultiplier[1];
  int RefClock_Reg[NUMBER_OF_REFCLKS];
  int TxCoreClock_Reg[1];
  int RxCoreClock_Reg[1];	
  int Channel_Reg[NUMBER_OF_LINKS];
  int Counter_1ms_Reg[NUMBER_OF_QUADS];
  float BER[NUMBER_OF_LINKS];
  float Totalbits_tx[NUMBER_OF_LINKS];
  float Totalbits_rx[NUMBER_OF_LINKS];	
  int Hours[NUMBER_OF_QUADS];
  int Minutes[NUMBER_OF_QUADS];
  int Seconds[NUMBER_OF_QUADS];
  int temp;
  int PrbsSelect[NUMBER_OF_LINKS];
  int PrbsPattern[NUMBER_OF_LINKS];

  int Locked[NUMBER_OF_LINKS];
  int Powerdown[NUMBER_OF_LINKS];
  int PLL_Locked[NUMBER_OF_LINKS];
  int Rx_FreqLocked[NUMBER_OF_LINKS];
  float ErrorCount[NUMBER_OF_LINKS]; 
  int ErrorCount_Reg_L[NUMBER_OF_LINKS];
  int ErrorCount_Reg_H[NUMBER_OF_LINKS];
  int Loopback[NUMBER_OF_LINKS];
  int tx_vodctrl[NUMBER_OF_LINKS];
  int tx_preemp_0t[NUMBER_OF_LINKS];
  int tx_preemp_1t[NUMBER_OF_LINKS];
  int tx_preemp_2t[NUMBER_OF_LINKS];
  int rx_eqdcgain[NUMBER_OF_LINKS];
  int rx_eqctrl[NUMBER_OF_LINKS];  
  int SelectedChannel;
  int i,j;
  int Show_ErrorCount;
  int BERInterval;
  int TimeInterval;
  int Reg_data;
  int Ravail;
  int High_nibble;
  int Low_nibble;
  int Rev_Serial[NUMBER_OF_LINKS];
  int Rev_Parallel_Available[NUMBER_OF_LINKS];
  int Rev_Parallel[NUMBER_OF_LINKS];  
  int Loopback_All;

  int datarate_tx;
  int datarate_rx;
  int show_coreclks;
  int Refclk_select_tx;
  int Refclk_select_rx;
  int tx_reconfig_data[7];
  int rx_reconfig_data[3];
  
  float eye;
  int eye_phase;
  float BER_Minimum;
  int BER_Minimum_phase;
  int start_value;
  int end_value;
  int optimum_phase;
  int table_phase_start;
  int index;  
  
  int eye1d;
  int eyeqbw;
  int eyeq_status;
  int eyeq_control;
  
  float ErrorCount_Array[64];
  int Table[64][13];  
  float BER_Array[64];
  
  int readback;
		
  
  //char line[100]; 


    
  /**********************************************************************/
  /*  Initialize all PMA Settings                                      */
  /**********************************************************************/
  
  for (i = 0; i < NUMBER_OF_LINKS ; i++)
  { 
  tx_vodctrl[i]    = VOD_800MV; 
  tx_preemp_0t[i]  = PRE_TAP_0;
  tx_preemp_1t[i]  = POST_TAP1_0; 
  tx_preemp_2t[i]  = POST_TAP2_0;
  rx_eqdcgain[i]   = EQ_GAIN_0;
  rx_eqctrl[i]     = EQ_CTRL_0;

// printf("\nValue before PMA configuration");
// readback = read_reconfiguration_direct_register_mode_1(i,0x3);
// readback = read_reconfiguration_direct_register_mode_1(i,0x5);
	  
  write_reconfiguration(i+4,VOD,tx_vodctrl[i]); //Tx only
  write_reconfiguration(i+4,PRE_TAP,tx_preemp_0t[i]); //Tx only
  write_reconfiguration(i+4,POST_TAP1,tx_preemp_1t[i]);	//Tx only  
  write_reconfiguration(i+4,POST_TAP2,tx_preemp_2t[i]);	//Tx only
  write_reconfiguration(i,EQ_GAIN,rx_eqdcgain[i]);	 
  write_reconfiguration(i,EQ_CTRL,rx_eqctrl[i]);	

// printf("\nValue after PMA configuration");	  
// readback = read_reconfiguration_direct_register_mode_1(i,0x3);
// readback = read_reconfiguration_direct_register_mode_1(i,0x5);	 

  

  Loopback[i] = 0;  	  
  Control_Reg[i] = 0x0000;      
  Control2_Reg[i] = 0x0000;
  Powerdown[i] = 0;
  Rev_Parallel[i] = 0;  
    
  } 
    Loopback_All		  = 0x0000;  //Disable Serial Loopback by default 
    //write_Loopback(Loopback_All);
  
  Refclk_select_tx = TX_5400MBPS_REFCLK;
  Refclk_select_rx = RX_5400MBPS_REFCLK;
	tx_reconfig_data[0] = 0 ; // Not used
	tx_reconfig_data[1] = 0 ; // Not used
	tx_reconfig_data[2] = 0 ; // Not used
	tx_reconfig_data[3] = PLL_5400MBPS_4 ;
	tx_reconfig_data[4] = PLL_5400MBPS_6 ;
	tx_reconfig_data[5] = PLL_5400MBPS_7 ;
	rx_reconfig_data[0] = RX_5400MBPS_4 ;
	//rx_reconfig_data[1] = RX_5400MBPS_13 ;
	//rx_reconfig_data[2] = RX_5400MBPS_14 ;
  
//  Refclk_select_tx = TX_2970MBPS_REFCLK;
//  Refclk_select_rx = RX_2970MBPS_REFCLK;
//	tx_reconfig_data[0] = TX_2970MBPS_4 ;
//	tx_reconfig_data[1] = TX_2970MBPS_6 ;
//	tx_reconfig_data[2] = TX_2970MBPS_8 ;
//	tx_reconfig_data[3] = PLL_2970MBPS_4 ;
//	tx_reconfig_data[4] = PLL_2970MBPS_6 ;
//	tx_reconfig_data[5] = PLL_2970MBPS_7 ;
//	rx_reconfig_data[0] = RX_2970MBPS_4 ;
 
  
  /**********************************************************************/
  /*  Set PRBS-Pattern to PRBS23                                        */
  /**********************************************************************/  
  
    
  for (i = 0; i < NUMBER_OF_LINKS ; i++)
  {
	  PrbsSelect[i] = 0x1; 
  }

     for (i = 0; i < NUMBER_OF_LINKS ; i++)
     {
     Control2_Reg[0] = (Control2_Reg[0] & (0xFFFC)) | ( i ); // Select channel
     Control2_Reg[0] = Control2_Reg[0] | (0x1000); /* Latch PrbsPattern Assert */
     Control2_Reg[0] = (Control2_Reg[0] & (0xFCFF)) | (PrbsSelect[i] << 8); 
     IOWR_ALTERA_AVALON_PIO_DATA(CONTROL2_REG_BASE,Control2_Reg[0]);
     
     Control2_Reg[0] = Control2_Reg[0] & (0xEFFF); /* Latch PrbsPattern Deassert */
     IOWR_ALTERA_AVALON_PIO_DATA(CONTROL2_REG_BASE,Control2_Reg[0]);
     }
     
     Control2_Reg[0] = Control2_Reg[0] & (0xFFFC); 
     IOWR_ALTERA_AVALON_PIO_DATA(CONTROL2_REG_BASE,Control2_Reg[0]);
     

  
  SelectedChannel = 0; // Default Selected Channels is channel 0 

  

  datarate_tx = RATE_5400MBPS; // This is how the design comes up.
  datarate_rx = RATE_5400MBPS;

//  datarate_tx = RATE_2970MBPS; // This is how the design comes up.
//  datarate_rx = RATE_2970MBPS;	  
  
  /**********************************************************************/
  /*  Reset All Channels                                                  */
  /**********************************************************************/  
  
  for (i = 0; i < NUMBER_OF_QUADS ; i++)
        {
        Control_Reg[i] = Control_Reg[i] | (0x8000);
        }
        IOWR_ALTERA_AVALON_PIO_DATA(CONTROL_REG_BASE,Control_Reg[0]);

        usleep(10);
        
        for (i = 0; i < NUMBER_OF_QUADS ; i++)
        {      
        Control_Reg[i] = Control_Reg[i] & (0x7FFF);
        }
        IOWR_ALTERA_AVALON_PIO_DATA(CONTROL_REG_BASE,Control_Reg[0]);

    
   
   
   Show_ErrorCount = 0;    
   
   BERInterval = 1; // Set to 2 seconds as default; 
   
   // Alternative to usleep 
   //do
   // {
   //  Counter_1ms_Reg[0] = IORD_ALTERA_AVALON_PIO_DATA(COUNTER_1MS_REG_BASE); // NUMBER_OF_MS_PER_SECOND;
   //} while (Counter_1ms_Reg[0] < 1000);
   
   usleep(2000000); // Wait around 2 seconds for reference clock measurement
   
   show_coreclks = 1;
  /**********************************************************************/
  /*  Main Program Loop                                                 */
  /**********************************************************************/  
  while (1)
	{
 
      
  Channel_Reg[0]      = IORD_ALTERA_AVALON_PIO_DATA(CHANNEL0_REG_BASE);
  Channel_Reg[1]      = IORD_ALTERA_AVALON_PIO_DATA(CHANNEL1_REG_BASE);
  Channel_Reg[2]      = IORD_ALTERA_AVALON_PIO_DATA(CHANNEL2_REG_BASE);
  Channel_Reg[3]      = IORD_ALTERA_AVALON_PIO_DATA(CHANNEL3_REG_BASE);

  RefClock_Reg[0]   = IORD_ALTERA_AVALON_PIO_DATA(REFCLOCK0_REG_BASE);  
  RefClock_Reg[1]   = IORD_ALTERA_AVALON_PIO_DATA(REFCLOCK1_REG_BASE); 	
  RefClock_Reg[2]   = IORD_ALTERA_AVALON_PIO_DATA(REFCLOCK2_REG_BASE);		
		
  TxCoreClock_Reg[0]  = IORD_ALTERA_AVALON_PIO_DATA(TXCORECLK_REG_BASE); 	
  RxCoreClock_Reg[0]  = IORD_ALTERA_AVALON_PIO_DATA(RXCORECLK_REG_BASE); 	

  Refclockmultiplier[0] = (0x003F) & (Channel_Reg[0] >> 6);	
  Bitrate_temp[RATE_5400MBPS] =  ((float)RefClock_Reg[0]* Refclockmultiplier[0] * 2)/(1000000);
  Bitrate_temp[RATE_2700MBPS] =  ((float)RefClock_Reg[0]* Refclockmultiplier[0])/(1000000);	
  Bitrate_temp[RATE_1620MBPS] =  ((float)RefClock_Reg[0]* 12)/(1000000);			
  Bitrate_temp[RATE_2970MBPS] =  ((float)RefClock_Reg[2]* 10)/(1000000);	
		
  for (i = 0; i < NUMBER_OF_DATARATES ; i++)
  {  	
  Bitrate[i] = (Bitrate_temp[i]); // Convert the floats to integer numbers
  }
  
  Counter_1ms_Reg[0]     = IORD_ALTERA_AVALON_PIO_DATA(COUNTER_1MS_REG_BASE);
 
  ErrorCount_Reg_L[0]	 = IORD_ALTERA_AVALON_PIO_DATA(ERRORCOUNT_L_0_BASE);
  ErrorCount_Reg_H[0]	 = IORD_ALTERA_AVALON_PIO_DATA(ERRORCOUNT_H_0_BASE);
  ErrorCount_Reg_L[1]	 = IORD_ALTERA_AVALON_PIO_DATA(ERRORCOUNT_L_1_BASE);
  ErrorCount_Reg_H[1]	 = IORD_ALTERA_AVALON_PIO_DATA(ERRORCOUNT_H_1_BASE);
  ErrorCount_Reg_L[2]	 = IORD_ALTERA_AVALON_PIO_DATA(ERRORCOUNT_L_2_BASE);
  ErrorCount_Reg_H[2]	 = IORD_ALTERA_AVALON_PIO_DATA(ERRORCOUNT_H_2_BASE);
  ErrorCount_Reg_L[3]	 = IORD_ALTERA_AVALON_PIO_DATA(ERRORCOUNT_L_3_BASE);
  ErrorCount_Reg_H[3]	 = IORD_ALTERA_AVALON_PIO_DATA(ERRORCOUNT_H_3_BASE);

  
  
  for (i = 0; i < NUMBER_OF_LINKS ; i++)
  {
  ErrorCount[i] = ((float) (ErrorCount_Reg_H[i]) *  MULTIPLIER * MULTIPLIER) + (float) (ErrorCount_Reg_L[i]);
  }

  
  for (i = 0; i < NUMBER_OF_LINKS ; i++)
  {
  Locked[i]  = 0x0001 & (Channel_Reg[i] >> 15);    
  PLL_Locked[i]  = 0x0001 & (Channel_Reg[i] >> 14);
  Rev_Serial[i]  = 0x0001 & (Channel_Reg[i] >> 13);  
  Rx_FreqLocked[i]   = 0x0001 & (Channel_Reg[i] >> 12); 
  Rev_Parallel_Available[i] = 0x0001 & (Channel_Reg[i]);   
  //if (Locked[i] == 1)
  //  ErrorCount[i] = 0x007F & (Channel_Reg[i]);
  //else
  //  ErrorCount[i] = 99;
  // 
  Totalbits_tx[i] = (float) (Counter_1ms_Reg[0]) * (float) (Bitrate[datarate_tx]*1000);	  
  Totalbits_rx[i] = (float) (Counter_1ms_Reg[0]) * (float) (Bitrate[datarate_rx]*1000);	 	  
  }
  

  
  for (i = 0; i < NUMBER_OF_QUADS ; i++)
  {
  Hours[i] =  Counter_1ms_Reg[i] / NUMBER_OF_MS_PER_HOUR ;
  Minutes[i] = (Counter_1ms_Reg[i] - (Hours[i] * NUMBER_OF_MS_PER_HOUR))/NUMBER_OF_MS_PER_MINUTE;
  Seconds[i] = (Counter_1ms_Reg[i] - (Hours[i] * NUMBER_OF_MS_PER_HOUR) - (Minutes[i] * NUMBER_OF_MS_PER_MINUTE)) / NUMBER_OF_MS_PER_SECOND;
  }
  for (i = 0; i < NUMBER_OF_LINKS ; i++)
  {
      if (ErrorCount[i] > 0)
        BER[i] = ((ErrorCount[i])) / Totalbits_tx[i];
      else
        //BER[i] = 0;
        BER[i] = ((float) 3) / Totalbits_tx[i]; /* confidence level (CL) of 95% = -ln(1-CL) = 3*/
  } /* For Loop */  
  
  for (i = 0; i < NUMBER_OF_LINKS ; i++)
  {
    switch(PrbsSelect[i])
    {
      case 0 : PrbsPattern[i] = 7;break;
      case 1 : PrbsPattern[i] = 23;break;
      case 2 : PrbsPattern[i] = 31;break;
      case 3 : PrbsPattern[i] = 15;break;
      default:break;
    }
  } /* For Loop */    



    printf("\n\n\n");
    printf("Stratix V Dynamic Datarate switching desing using %2d Ch \n",NUMBER_OF_LINKS);
    printf("--------------------------------------------------\n");
    printf("|0:GXB_RXTX[12] SMA  PRBS-%2d Tx Bitrate %4d Mbps Rx Bitrate %4d Mbps |\n",PrbsPattern[0],Bitrate[datarate_tx],Bitrate[datarate_rx]);
    printf("|1:GXB_RXTX[13] SMA  PRBS-%2d Tx Bitrate %4d Mbps Rx Bitrate %4d Mbps |\n",PrbsPattern[1],Bitrate[datarate_tx],Bitrate[datarate_rx]);
    printf("|2:GXB_RXTX[14] SMA  PRBS-%2d Tx Bitrate %4d Mbps Rx Bitrate %4d Mbps |\n",PrbsPattern[2],Bitrate[datarate_tx],Bitrate[datarate_rx]);
    printf("|3:GXB_RXTX[15] SMA  PRBS-%2d Tx Bitrate %4d Mbps Rx Bitrate %4d Mbps |\n",PrbsPattern[3],Bitrate[datarate_tx],Bitrate[datarate_rx]);    
    printf("--------------------------------------------------\n");
    //printf("Serial LoopBack : %d\n",Loopback);
    printf("SelectedChannel: ");
     switch (SelectedChannel)
     {
       case  0  : printf(" 0 :GXB_RXTX[12] SMA  PRBS-%2d Tx Bitrate %4d Mbps Rx Bitrate %4d Mbps |\n",PrbsPattern[0],Bitrate[datarate_tx],Bitrate[datarate_rx]);break;
       case  1  : printf(" 1 :GXB_RXTX[13] SMA  PRBS-%2d Tx Bitrate %4d Mbps Rx Bitrate %4d Mbps |\n",PrbsPattern[1],Bitrate[datarate_tx],Bitrate[datarate_rx]);break;
       case  2  : printf(" 2 :GXB_RXTX[14] SMA  PRBS-%2d Tx Bitrate %4d Mbps Rx Bitrate %4d Mbps |\n",PrbsPattern[2],Bitrate[datarate_tx],Bitrate[datarate_rx]);break;
       case  3  : printf(" 3 :GXB_RXTX[15] SMA  PRBS-%2d Tx Bitrate %4d Mbps Rx Bitrate %4d Mbps |\n",PrbsPattern[3],Bitrate[datarate_tx],Bitrate[datarate_rx]);break;
      default : break;
     }  
    printf("\n");

     
     printf("Channel            :|");
     for (i = 0; i < NUMBER_OF_LINKS ; i++)
     { 
       printf("%2X|",i);
     }
     printf("\n");

     printf("                    |");
     for (i = 0; i < NUMBER_OF_LINKS ; i++)
     { 
     printf("--|");
     }
     printf("\n");

     printf("PowerDown          :|");        
     for (i = 0; i < NUMBER_OF_LINKS ; i++)
     { 
     printf("%2d|",Powerdown[i]);
     }
     printf("\n");

     printf("Rev. Serial Loop   :|");        
     for (i = 0; i < NUMBER_OF_LINKS ; i++)
     { 
 	if (Rev_Serial[i] == 1)
       	printf("%2d|",Rev_Serial[i]);
 	else
 		printf("  |");
     }
     printf("\n");
     
     printf("Rev. Parallel Loop :|");        
     for (i = 0; i < NUMBER_OF_LINKS ; i++)
     { 
     if (Rev_Parallel_Available[i] == 1)
 	  printf("%2d|",Rev_Parallel[i]);
     else
 	  printf("  |");
     }
     printf("\n");
     
//     printf("Serial Loop        :|");        
//     for (i = 0; i < NUMBER_OF_LINKS ; i++)
//     { 
//     if (Powerdown[i] == 0)
//       printf("%2d|",Loopback[i]);
//     else
//       printf("  |");   
//     }
//     printf("\n");
        
     printf("PLLLocked          :|");        
     for (i = 0; i < NUMBER_OF_LINKS ; i++)
     {
     if (Powerdown[i] == 0) 
       printf("%2d|",PLL_Locked[i]);
     else
       printf("  |");    
     }
     printf("\n"); 
     
     printf("FreqLocked         :|");        
     for (i = 0; i < NUMBER_OF_LINKS ; i++)
     {
     if (Powerdown[i] == 0)       
       printf("%2d|",Rx_FreqLocked[i]);
     else
       printf("  |");    
     }
     printf("\n");    
     
     printf("PrbsLocked         :|");

     for (i = 0; i < NUMBER_OF_LINKS ; i++)
     {
     if (Powerdown[i] == 0)      
       printf("%2d|",Locked[i]);
     else
       printf("  |");
     }
     printf("\n");
   
       
     printf("Errorcount         :|");
     for (i = 0; i < NUMBER_OF_LINKS ; i++)
     { 
     if ((Locked[i] == 0) || (Powerdown[i] == 1))
       printf("  |");
     else if (ErrorCount[i] > 99)
       printf("xx|");
     else	
       printf("%2d|",ErrorCount_Reg_L[i]);      
     }
     printf("\n\n");
    
    for (i = 0; i < NUMBER_OF_LINKS ; i++)
    { 
      if (Powerdown[i] == 1)
        printf("Ch%2d Powered down\n",i);        
      else if (Locked[i] == 1)
      {
 	    if (ErrorCount[i] > 0)
              printf("BER Estimate  Ch %1X  : %e    (# Errors/Totalbits)\n",i,BER[i]);
          else
              printf("BER (CL=0.95) Ch %1X  : %e\n",i,BER[i]);
      }
      else
        printf("No Data Lock on Ch%2d \n",i);
    }

    for (i = 0; i < NUMBER_OF_LINKS ; i++)
    { 
      if (Show_ErrorCount  == 1)
        {
         printf("\n");
         if (Powerdown[i] == 1)
             printf("Ch%2d Powered down\n",i);        
         else if (Locked[i] == 1)    
        {
             printf("ErrorCount  Ch %1X  : %e    \n",i,ErrorCount[i]);
        }
        else
          printf("No Data Lock on Ch%2d \n",i);
        }
    }
    
    Show_ErrorCount = 0;
    
     printf("\n");
     
     if (Powerdown[0] == 0) 
        printf("Test Time             : %2dh %2dm %2ds\n",Hours[0],Minutes[0],Seconds[0]);                
    
    printf("Reference Clock 0  Frequency  : %7d kHz\n",RefClock_Reg[0]/1000);
    printf("Reference Clock 1  Frequency  : %7d kHz\n",RefClock_Reg[2]/1000);
    //printf("Reference Clock 2  Frequency  : %7d kHz\n",RefClock_Reg[2]/1000);	  
	 if (show_coreclks == 1)
	 {
		 printf("TxCoreClk Frequency           : %7d kHz\n",TxCoreClock_Reg[0]/1000);
		 printf("RxCoreClk Frequency           : %7d kHz\n",RxCoreClock_Reg[0]/1000); 
	 }
	 show_coreclks = 1;
	 //printf("Selected Reference clock Tx   : %1d\n",datarate_tx);
	 //printf("Selected Reference clock Rx   : %1d\n",datarate_rx);
    printf("\nSelect Action : \n");
    printf("===============\n");
    
    printf("1. Start Test or Do Full Reset on all Channels\n");
    printf("3. Select Channel To Control\n");
    printf("4. Show/Control Transceiver PMA Settings on Selected Channel\n");
    printf("5. Insert Biterror on Selected Channel\n");
    printf("6. Reset ErrorCounter on Selected Channel\n");
    printf("7. Show Status\n");
    //printf("8. Toggle SerialLoopback on Selected Channel and Reset\n");
    printf("A. Select Prbs Pattern on Selected Channel\n");
    printf("B. Show detailed Errorcount\n");
    printf("C. Refresh BER every %d seconds on selected channel\n",BERInterval);
    printf("D. Input new BER Time Interval\n");
    printf("E. Show Transceiver PMA Settings on all channels\n");
	 printf("M. Measure 1-D eye on selected channel\n");
    if (Rev_Parallel_Available[0] == 1) 
	    printf("F. Toggle Parallel Loopback on selected channel\n");  
//	 printf("R. Change Datarate Rx using Hardware and reset \n");
//    printf("T. Change Datarate Tx using Hardware and reset \n");
    printf("R. Change Datarate Rx using Software and reset \n");
	 printf("T. Change Datarate Tx using Software and reset \n");
	 printf("0. Stop Test \n\n");
    printf("  Enter Choice :");
  
    rx_char = input_char(); 
    
    
		switch (rx_char)
		{
      case '1': /*Start/Reset All Quads*/
        for (i = 0; i < NUMBER_OF_QUADS ; i++)
        {
        Control_Reg[i] = Control_Reg[i] | (0x8000);
        }
        IOWR_ALTERA_AVALON_PIO_DATA(CONTROL_REG_BASE,Control_Reg[0]);

        usleep(10);
        
        for (i = 0; i < NUMBER_OF_QUADS ; i++)
        {      
        Control_Reg[i] = Control_Reg[i] & (0x7FFF);
        }
        IOWR_ALTERA_AVALON_PIO_DATA(CONTROL_REG_BASE,Control_Reg[0]);
        
        
        usleep(1000000); // Wait for 1 second
        printf("All Channels resetted and tests started.\n\n\n");
        break; //case '1'

                
      case '3': /*Select Channel To Control*/

      printf("3. Select Channel To Control\n");
      printf("Current Channel : ");
      switch (SelectedChannel)
       {
       case  0  : printf(" 0 :GXB_RXTX[12] SMA  PRBS-%2d Tx Bitrate %4d Mbps Rx Bitrate %4d Mbps |\n",PrbsPattern[0],Bitrate[datarate_tx],Bitrate[datarate_rx]);break;
       case  1  : printf(" 1 :GXB_RXTX[13] SMA  PRBS-%2d Tx Bitrate %4d Mbps Rx Bitrate %4d Mbps |\n",PrbsPattern[1],Bitrate[datarate_tx],Bitrate[datarate_rx]);break;
       case  2  : printf(" 2 :GXB_RXTX[14] SMA  PRBS-%2d Tx Bitrate %4d Mbps Rx Bitrate %4d Mbps |\n",PrbsPattern[2],Bitrate[datarate_tx],Bitrate[datarate_rx]);break;
       case  3  : printf(" 3 :GXB_RXTX[15] SMA  PRBS-%2d Tx Bitrate %4d Mbps Rx Bitrate %4d Mbps |\n",PrbsPattern[3],Bitrate[datarate_tx],Bitrate[datarate_rx]);break;

       default : break;
       }  
     
     temp = 0xffffffff;
  
     do {
      printf("New Channel (0-3)         :");
      rx_char = input_char(); 
  
      switch (rx_char)
      {
      case '0': temp = 0; break;
      case '1': temp = 1; break;
      case '2': temp = 2; break;
      case '3': temp = 3; break;
      default : break;
      }
  
    } while (temp == 0xffffffff);
    
    SelectedChannel = temp;
    
    
          /*printf("temp %d : \n",temp);
          printf("SelectedChannel %d : \n",SelectedChannel);*/
          Control2_Reg[0] = (Control2_Reg[0] & (0xFFF8)) | (SelectedChannel);
          IOWR_ALTERA_AVALON_PIO_DATA(CONTROL2_REG_BASE,Control2_Reg[0]);      
                   

        
      break; //case '3'
                
			case '4': /*Show/Control Transceiver Parameters */

        printf("4. Transceiver PMA Settings of Channel %1d\n",SelectedChannel);
           switch (tx_vodctrl[SelectedChannel])
           {
           case  VOD_200MV  : printf("    VOD Level           :  200mV\n"); break;
           case  VOD_400MV  : printf("    VOD Level           :  400mV\n"); break;
           case  VOD_600MV  : printf("    VOD Level           :  600mV\n"); break;
           case  VOD_700MV  : printf("    VOD Level           :  700mV\n"); break;           
           case  VOD_800MV  : printf("    VOD Level           :  800mV\n"); break;
           case  VOD_900MV  : printf("    VOD Level           :  900mV\n"); break;           
           case  VOD_1000MV : printf("    VOD Level           : 1000mV\n"); break;
           case  VOD_1200MV : printf("    VOD Level           : 1200mV\n"); break;                                                              
           default : break;
           }
   
           
           switch (tx_preemp_0t[SelectedChannel])
           {
           case  PRE_TAP_15N : printf("    Pre Tap Level       : -15\n"); break;
           case  PRE_TAP_14N : printf("    Pre Tap Level       : -14\n"); break;
           case  PRE_TAP_13N : printf("    Pre Tap Level       : -13\n"); break;
           case  PRE_TAP_12N : printf("    Pre Tap Level       : -12\n"); break;
           case  PRE_TAP_11N : printf("    Pre Tap Level       : -11\n"); break;
           case  PRE_TAP_10N : printf("    Pre Tap Level       : -10\n"); break;
           case  PRE_TAP_9N  : printf("    Pre Tap Level       : -9\n"); break;  
           case  PRE_TAP_8N  : printf("    Pre Tap Level       : -8\n"); break;              
           case  PRE_TAP_7N  : printf("    Pre Tap Level       : -7\n"); break;
           case  PRE_TAP_6N  : printf("    Pre Tap Level       : -6\n"); break;
           case  PRE_TAP_5N  : printf("    Pre Tap Level       : -5\n"); break;
           case  PRE_TAP_4N  : printf("    Pre Tap Level       : -4\n"); break;
           case  PRE_TAP_3N  : printf("    Pre Tap Level       : -3\n"); break;
           case  PRE_TAP_2N  : printf("    Pre Tap Level       : -2\n"); break;
           case  PRE_TAP_1N  : printf("    Pre Tap Level       : -1\n"); break;
           case  PRE_TAP_0   : printf("    Pre Tap Level       :  0\n"); break;
           case  PRE_TAP_1   : printf("    Pre Tap Level       :  1\n"); break;
           case  PRE_TAP_2   : printf("    Pre Tap Level       :  2\n"); break;
           case  PRE_TAP_3   : printf("    Pre Tap Level       :  3\n"); break;
           case  PRE_TAP_4   : printf("    Pre Tap Level       :  4\n"); break;
           case  PRE_TAP_5   : printf("    Pre Tap Level       :  5\n"); break;
           case  PRE_TAP_6   : printf("    Pre Tap Level       :  6\n"); break;
           case  PRE_TAP_7   : printf("    Pre Tap Level       :  7\n"); break; 
           case  PRE_TAP_8   : printf("    Pre Tap Level       :  8\n"); break;
           case  PRE_TAP_9   : printf("    Pre Tap Level       :  9\n"); break;
           case  PRE_TAP_10  : printf("    Pre Tap Level       :  10\n"); break;
           case  PRE_TAP_11  : printf("    Pre Tap Level       :  11\n"); break;
           case  PRE_TAP_12  : printf("    Pre Tap Level       :  12\n"); break;
           case  PRE_TAP_13  : printf("    Pre Tap Level       :  13\n"); break;
           case  PRE_TAP_14  : printf("    Pre Tap Level       :  14\n"); break;
           case  PRE_TAP_15  : printf("    Pre Tap Level       :  15\n"); break;               
           default : break;
           }

           switch (tx_preemp_1t[SelectedChannel])
           {
           case  POST_TAP1_0  : printf("    Post Tap 1 Level    :  0\n"); break;
           case  POST_TAP1_1  : printf("    Post Tap 1 Level    :  1\n"); break;    
           case  POST_TAP1_2  : printf("    Post Tap 1 Level    :  2\n"); break;
           case  POST_TAP1_3  : printf("    Post Tap 1 Level    :  3\n"); break;   
           case  POST_TAP1_4  : printf("    Post Tap 1 Level    :  4\n"); break;
           case  POST_TAP1_5  : printf("    Post Tap 1 Level    :  5\n"); break;   
           case  POST_TAP1_6  : printf("    Post Tap 1 Level    :  6\n"); break;
           case  POST_TAP1_7  : printf("    Post Tap 1 Level    :  7\n"); break;   
           case  POST_TAP1_8  : printf("    Post Tap 1 Level    :  8\n"); break;
           case  POST_TAP1_9  : printf("    Post Tap 1 Level    :  9\n"); break;   
           case  POST_TAP1_10 : printf("    Post Tap 1 Level    : 10\n"); break;
           case  POST_TAP1_11 : printf("    Post Tap 1 Level    : 11\n"); break;   
           case  POST_TAP1_12 : printf("    Post Tap 1 Level    : 12\n"); break;     
           case  POST_TAP1_13 : printf("    Post Tap 1 Level    : 13\n"); break;   
           case  POST_TAP1_14 : printf("    Post Tap 1 Level    : 14\n"); break;
           case  POST_TAP1_15 : printf("    Post Tap 1 Level    : 15\n"); break;   
           case  POST_TAP1_16 : printf("    Post Tap 1 Level    : 16\n"); break;
           case  POST_TAP1_17 : printf("    Post Tap 1 Level    : 17\n"); break;   
           case  POST_TAP1_18 : printf("    Post Tap 1 Level    : 18\n"); break;
           case  POST_TAP1_19 : printf("    Post Tap 1 Level    : 19\n"); break;   
           case  POST_TAP1_20 : printf("    Post Tap 1 Level    : 20\n"); break;
           case  POST_TAP1_21 : printf("    Post Tap 1 Level    : 21\n"); break;   
           case  POST_TAP1_22 : printf("    Post Tap 1 Level    : 22\n"); break; 
           case  POST_TAP1_23 : printf("    Post Tap 1 Level    : 23\n"); break;   
           case  POST_TAP1_24 : printf("    Post Tap 1 Level    : 24\n"); break;
           case  POST_TAP1_25 : printf("    Post Tap 1 Level    : 25\n"); break;   
           case  POST_TAP1_26 : printf("    Post Tap 1 Level    : 26\n"); break;
           case  POST_TAP1_27 : printf("    Post Tap 1 Level    : 27\n"); break;   
           case  POST_TAP1_28 : printf("    Post Tap 1 Level    : 28\n"); break;
           case  POST_TAP1_29 : printf("    Post Tap 1 Level    : 29\n"); break;   
           case  POST_TAP1_30 : printf("    Post Tap 1 Level    : 30\n"); break;
           case  POST_TAP1_31 : printf("    Post Tap 1 Level    : 31\n"); break;             
           default : break;
           }

           switch (tx_preemp_2t[SelectedChannel])
           {
           case  POST_TAP2_15N : printf("    Post Tap 2 Level       : -15\n"); break;
           case  POST_TAP2_14N : printf("    Post Tap 2 Level       : -14\n"); break;
           case  POST_TAP2_13N : printf("    Post Tap 2 Level       : -13\n"); break;
           case  POST_TAP2_12N : printf("    Post Tap 2 Level       : -12\n"); break;
           case  POST_TAP2_11N : printf("    Post Tap 2 Level       : -11\n"); break;
           case  POST_TAP2_10N : printf("    Post Tap 2 Level       : -10\n"); break;
           case  POST_TAP2_9N  : printf("    Post Tap 2 Level       : -9\n"); break;  
           case  POST_TAP2_8N  : printf("    Post Tap 2 Level       : -8\n"); break;              
           case  POST_TAP2_7N  : printf("    Post Tap 2 Level       : -7\n"); break;
           case  POST_TAP2_6N  : printf("    Post Tap 2 Level       : -6\n"); break;
           case  POST_TAP2_5N  : printf("    Post Tap 2 Level       : -5\n"); break;
           case  POST_TAP2_4N  : printf("    Post Tap 2 Level       : -4\n"); break;
           case  POST_TAP2_3N  : printf("    Post Tap 2 Level       : -3\n"); break;
           case  POST_TAP2_2N  : printf("    Post Tap 2 Level       : -2\n"); break;
           case  POST_TAP2_1N  : printf("    Post Tap 2 Level       : -1\n"); break;
           case  POST_TAP2_0   : printf("    Post Tap 2 Level       :  0\n"); break;
           case  POST_TAP2_1   : printf("    Post Tap 2 Level       :  1\n"); break;
           case  POST_TAP2_2   : printf("    Post Tap 2 Level       :  2\n"); break;
           case  POST_TAP2_3   : printf("    Post Tap 2 Level       :  3\n"); break;
           case  POST_TAP2_4   : printf("    Post Tap 2 Level       :  4\n"); break;
           case  POST_TAP2_5   : printf("    Post Tap 2 Level       :  5\n"); break;
           case  POST_TAP2_6   : printf("    Post Tap 2 Level       :  6\n"); break;
           case  POST_TAP2_7   : printf("    Post Tap 2 Level       :  7\n"); break; 
           case  POST_TAP2_8   : printf("    Post Tap 2 Level       :  8\n"); break;
           case  POST_TAP2_9   : printf("    Post Tap 2 Level       :  9\n"); break;
           case  POST_TAP2_10  : printf("    Post Tap 2 Level       :  10\n"); break;
           case  POST_TAP2_11  : printf("    Post Tap 2 Level       :  11\n"); break;
           case  POST_TAP2_12  : printf("    Post Tap 2 Level       :  12\n"); break;
           case  POST_TAP2_13  : printf("    Post Tap 2 Level       :  13\n"); break;
           case  POST_TAP2_14  : printf("    Post Tap 2 Level       :  14\n"); break;
                                                                                                                                                                                                           
           default : break;
           }

           switch (rx_eqdcgain[SelectedChannel])
           {
           case  EQ_GAIN_0    : printf("    Equalizer Gain Level    :  0\n"); break;
           case  EQ_GAIN_1    : printf("    Equalizer Gain Level    :  1\n"); break;
           case  EQ_GAIN_2    : printf("    Equalizer Gain Level    :  2\n"); break;   
           case  EQ_GAIN_3    : printf("    Equalizer Gain Level    :  3\n"); break;
           case  EQ_GAIN_4    : printf("    Equalizer Gain Level    :  4\n"); break;
           case  EQ_GAIN_5    : printf("    Equalizer Gain Level    :  5\n"); break;   
           case  EQ_GAIN_6    : printf("    Equalizer Gain Level    :  6\n"); break;
           case  EQ_GAIN_7    : printf("    Equalizer Gain Level    :  7\n"); break;
				  
           default : break;
           }

           switch (rx_eqctrl[SelectedChannel])
           {
           case  EQ_CTRL_0    : printf("    Equalizer Control Level :  0\n"); break;
           case  EQ_CTRL_1    : printf("    Equalizer Control Level :  1\n"); break;
           case  EQ_CTRL_2    : printf("    Equalizer Control Level :  2\n"); break;
           case  EQ_CTRL_3    : printf("    Equalizer Control Level :  3\n"); break;
           case  EQ_CTRL_4    : printf("    Equalizer Control Level :  4\n"); break;
           case  EQ_CTRL_5    : printf("    Equalizer Control Level :  5\n"); break;
           case  EQ_CTRL_6    : printf("    Equalizer Control Level :  6\n"); break;
           case  EQ_CTRL_7    : printf("    Equalizer Control Level :  7\n"); break;
           case  EQ_CTRL_8    : printf("    Equalizer Control Level :  8\n"); break;
           case  EQ_CTRL_9    : printf("    Equalizer Control Level :  9\n"); break;
           case  EQ_CTRL_10   : printf("    Equalizer Control Level : 10\n"); break;
           case  EQ_CTRL_11   : printf("    Equalizer Control Level : 11\n"); break;
           case  EQ_CTRL_12   : printf("    Equalizer Control Level : 12\n"); break;
           case  EQ_CTRL_13   : printf("    Equalizer Control Level : 13\n"); break;
           case  EQ_CTRL_14   : printf("    Equalizer Control Level : 14\n"); break;   
           case  EQ_CTRL_15   : printf("    Equalizer Control Level : 15\n"); break;                                                                                                                                                                                                                                                                                    
           default : break;
           }

        do {
        printf("\n");  
        
        printf("    New VOD Level Channel %1d               \n",SelectedChannel);
        printf("    ======================               \n");
        printf("    For VOD Level =  200 mV choose '0' \n");
        printf("    For VOD Level =  400 mV choose '1' \n");
        printf("    For VOD Level =  600 mV choose '2' \n");
        printf("    For VOD Level =  700 mV choose '3' \n");        
        printf("    For VOD Level =  800 mV choose '4' \n");
        printf("    For VOD Level =  900 mV choose '5' \n");        
        printf("    For VOD Level = 1000 mV choose '6' \n");
        printf("    For VOD Level = 1200 mV choose '7' \n");
        printf("    No Change               choose 'X' \n\n");    
        printf("    Choice :");      
        
        temp = 0xffffffff;
                                             
        rx_char = input_char(); 
        switch (rx_char)
        {
        case '0': temp =  VOD_200MV; break;
        case '1': temp =  VOD_400MV; break;
        case '2': temp =  VOD_600MV; break;
        case '3': temp =  VOD_700MV; break;        
        case '4': temp =  VOD_800MV; break;
        case '5': temp =  VOD_900MV; break;        
        case '6': temp =  VOD_1000MV; break;
        case '7': temp =  VOD_1200MV; break;
        case 'x': 
        case 'X': temp =  tx_vodctrl[SelectedChannel];break;
        default : break;
        }
        } while (temp == 0xffffffff);
 
        tx_vodctrl[SelectedChannel] = temp; 
        
        printf("\n");  
        
        do {
        printf("    New Pre Tap Level Channel %1d               \n",SelectedChannel);
        printf("    =========================                \n");
          printf("    For Pre Tap =  -15 choose '1' \n");
          printf("    For Pre Tap =  -14 choose '2' \n");
          printf("    For Pre Tap =  -13 choose '3' \n");
          printf("    For Pre Tap =  -12 choose '4' \n");
          printf("    For Pre Tap =  -11 choose '5' \n");
          printf("    For Pre Tap =  -10 choose '6' \n");
          printf("    For Pre Tap =  -9  choose '7' \n");
          printf("    For Pre Tap =  -8  choose '8' \n");
          printf("    For Pre Tap =  -7  choose '9' \n"); 
          printf("    For Pre Tap =  -6  choose 'A' \n");          
          printf("    For Pre Tap =  -5  choose 'B' \n");
          printf("    For Pre Tap =  -4  choose 'C' \n");
          printf("    For Pre Tap =  -3  choose 'D' \n");
          printf("    For Pre Tap =  -2  choose 'E' \n");          
          printf("    For Pre Tap =  -1  choose 'F' \n");
          printf("    For Pre Tap =   0  choose '0' \n");
          printf("    For Pre Tap =   1  choose 'G' \n");        
          printf("    For Pre Tap =   2  choose 'H' \n");
          printf("    For Pre Tap =   3  choose 'I' \n");
          printf("    For Pre Tap =   4  choose 'J' \n");
          printf("    For Pre Tap =   5  choose 'K' \n");        
          printf("    For Pre Tap =   6  choose 'L' \n");
          printf("    For Pre Tap =   7  choose 'M' \n");
          printf("    For Pre Tap =   8  choose 'N' \n");
          printf("    For Pre Tap =   9  choose 'O' \n");
          printf("    For Pre Tap =  10  choose 'P' \n");
          printf("    For Pre Tap =  11  choose 'Q' \n");        
          printf("    For Pre Tap =  12  choose 'R' \n");
          printf("    For Pre Tap =  13  choose 'S' \n");  
          printf("    For Pre Tap =  14  choose 'T' \n");
          printf("    For Pre Tap =  15  choose 'U' \n");                                                                                                        
        printf("    No Change         choose 'X' \n\n");    
        printf("    Choice :");
        
        temp = 0xffffffff;
                                                   
        rx_char = input_char(); 
        switch (rx_char)
        {
        case '0': temp =  PRE_TAP_0; break;
        case '1': temp =  PRE_TAP_15N; break;
        case '2': temp =  PRE_TAP_14N; break;
        case '3': temp =  PRE_TAP_13N; break;
        case '4': temp =  PRE_TAP_12N; break;
        case '5': temp =  PRE_TAP_11N; break;
        case '6': temp =  PRE_TAP_10N; break;
        case '7': temp =  PRE_TAP_9N; break;
        case '8': temp =  PRE_TAP_8N; break;
        case '9': temp =  PRE_TAP_7N; break;
        case 'a':
        case 'A': temp =  PRE_TAP_6N; break;
        case 'b':
        case 'B': temp =  PRE_TAP_5N; break;
        case 'c':
        case 'C': temp =  PRE_TAP_4N; break;
        case 'd':
        case 'D': temp =  PRE_TAP_3N; break;
        case 'e':
        case 'E': temp =  PRE_TAP_2N; break;      
        case 'f':
        case 'F': temp =  PRE_TAP_1N; break;   
        case 'g':
        case 'G': temp =  PRE_TAP_1; break;  
        case 'h':
        case 'H': temp =  PRE_TAP_2; break;  
        case 'i':
        case 'I': temp =  PRE_TAP_3; break;  
        case 'j':
        case 'J': temp =  PRE_TAP_4; break;   
        case 'k':
        case 'K': temp =  PRE_TAP_5; break;  
        case 'l':
        case 'L': temp =  PRE_TAP_6; break;  
        case 'm':
        case 'M': temp =  PRE_TAP_7; break;  
        case 'n':
        case 'N': temp =  PRE_TAP_8; break;   
        case 'o':
        case 'O': temp =  PRE_TAP_9; break;  
        case 'p':
        case 'P': temp =  PRE_TAP_10; break;  
        case 'q':
        case 'Q': temp =  PRE_TAP_11; break;  
        case 'r':
        case 'R': temp =  PRE_TAP_12; break;   
        case 's':
        case 'S': temp =  PRE_TAP_13; break;  
        case 't':
        case 'T': temp =  PRE_TAP_14; break;  
        case 'u':
        case 'U': temp =  PRE_TAP_15; break;           
        case 'x': 
        case 'X': temp = tx_preemp_0t[SelectedChannel];break;
        default : break;
        }
        } while (temp == 0xffffffff);       
        
        tx_preemp_0t[SelectedChannel] = temp;
       

        printf("\n");  
        
        do {
        printf("    New Post Tap 1 Level Channel %1d               \n",SelectedChannel);
        printf("    =========================                \n");
          printf("    For Post Tap 1 =  0 choose '0' \n");        
          printf("    For Post Tap 1 =  1 choose '1' \n"); 
          printf("    For Post Tap 1 =  2 choose '2' \n"); 
          printf("    For Post Tap 1 =  3 choose '3' \n"); 
          printf("    For Post Tap 1 =  4 choose '4' \n");
          printf("    For Post Tap 1 =  5 choose '5' \n");
          printf("    For Post Tap 1 =  6 choose '6' \n");
          printf("    For Post Tap 1 =  7 choose '7' \n");
          printf("    For Post Tap 1 =  8 choose '8' \n");
          printf("    For Post Tap 1 =  9 choose '9' \n");
          printf("    For Post Tap 1 = 10 choose 'A' \n");          
          printf("    For Post Tap 1 = 11 choose 'B' \n");
          printf("    For Post Tap 1 = 12 choose 'C' \n");
          printf("    For Post Tap 1 = 13 choose 'D' \n");
          printf("    For Post Tap 1 = 14 choose 'E' \n");
          printf("    For Post Tap 1 = 15 choose 'F' \n");
          printf("    For Post Tap 1 = 16 choose 'G' \n");
          printf("    For Post Tap 1 = 17 choose 'H' \n");
          printf("    For Post Tap 1 = 18 choose 'I' \n");
          printf("    For Post Tap 1 = 19 choose 'J' \n");
          printf("    For Post Tap 1 = 20 choose 'K' \n");
          printf("    For Post Tap 1 = 21 choose 'L' \n"); 
          printf("    For Post Tap 1 = 22 choose 'M' \n");
          printf("    For Post Tap 1 = 23 choose 'N' \n");
          printf("    For Post Tap 1 = 24 choose 'O' \n");
          printf("    For Post Tap 1 = 25 choose 'P' \n");              
          printf("    For Post Tap 1 = 26 choose 'Q' \n");
          printf("    For Post Tap 1 = 27 choose 'R' \n");
          printf("    For Post Tap 1 = 28 choose 'S' \n");
          printf("    For Post Tap 1 = 29 choose 'T' \n");
          printf("    For Post Tap 1 = 30 choose 'U' \n");
          printf("    For Post Tap 1 = 31 choose 'V' \n");                                                                                                         
          printf("    No Change           choose 'X' \n\n");    
          printf("    Choice :");
        
        temp = 0xffffffff;
                                        
        rx_char   = input_char();
        switch (rx_char)
        {
        case '0': temp =  POST_TAP1_0; break;
        case '1': temp =  POST_TAP1_1; break;
        case '2': temp =  POST_TAP1_2; break;
        case '3': temp =  POST_TAP1_3; break;
        case '4': temp =  POST_TAP1_4; break;
        case '5': temp =  POST_TAP1_5; break;
        case '6': temp =  POST_TAP1_6; break;
        case '7': temp =  POST_TAP1_7; break;
        case '8': temp =  POST_TAP1_8; break;
        case '9': temp =  POST_TAP1_9; break;
        case 'a':
        case 'A': temp =  POST_TAP1_10; break;
        case 'b':
        case 'B': temp =  POST_TAP1_11; break;
        case 'c':
        case 'C': temp =  POST_TAP1_12; break;  
        case 'd':        
        case 'D': temp =  POST_TAP1_13; break;
        case 'e':        
        case 'E': temp =  POST_TAP1_14; break;
        case 'f':        
        case 'F': temp =  POST_TAP1_15; break;
        case 'g':
        case 'G': temp =  POST_TAP1_16; break;
        case 'h':
        case 'H': temp =  POST_TAP1_17; break;
        case 'i':
        case 'I': temp =  POST_TAP1_18; break;
        case 'j':
        case 'J': temp =  POST_TAP1_19; break;
        case 'k':
        case 'K': temp =  POST_TAP1_20; break;  
        case 'l':
        case 'L': temp =  POST_TAP1_21; break;
        case 'm':
        case 'M': temp =  POST_TAP1_22; break;  
        case 'n':        
        case 'N': temp =  POST_TAP1_23; break;
        case 'o':        
        case 'O': temp =  POST_TAP1_24; break;
        case 'p':        
        case 'P': temp =  POST_TAP1_25; break;
        case 'q':
        case 'Q': temp =  POST_TAP1_26; break;
        case 'r':
        case 'R': temp =  POST_TAP1_27; break;
        case 's':
        case 'S': temp =  POST_TAP1_28; break;
        case 't':
        case 'T': temp =  POST_TAP1_29; break;
        case 'u':
        case 'U': temp =  POST_TAP1_30; break;  
        case 'v':
        case 'V': temp =  POST_TAP1_31; break;          
        case 'x':         
        case 'X': temp = tx_preemp_1t[SelectedChannel];break;
        default : break;
        }
        } while (temp == 0xffffffff);           
        
        
        tx_preemp_1t[SelectedChannel] = temp;        
           
        printf("\n");  
        
        do {
        printf("    New Post Tap 2 Level Channel %1d               \n",SelectedChannel);
        printf("    =========================                \n");
          printf("    For Post Tap 2 =  -15 choose '1' \n");
          printf("    For Post Tap 2 =  -14 choose '2' \n");
          printf("    For Post Tap 2 =  -13 choose '3' \n");
          printf("    For Post Tap 2 =  -12 choose '4' \n");
          printf("    For Post Tap 2 =  -11 choose '5' \n");
          printf("    For Post Tap 2 =  -10 choose '6' \n");
          printf("    For Post Tap 2 =  -9  choose '7' \n");
          printf("    For Post Tap 2 =  -8  choose '8' \n");
          printf("    For Post Tap 2 =  -7  choose '9' \n"); 
          printf("    For Post Tap 2 =  -6  choose 'A' \n");          
          printf("    For Post Tap 2 =  -5  choose 'B' \n");
          printf("    For Post Tap 2 =  -4  choose 'C' \n");
          printf("    For Post Tap 2 =  -3  choose 'D' \n");
          printf("    For Post Tap 2 =  -2  choose 'E' \n");          
          printf("    For Post Tap 2 =  -1  choose 'F' \n");
          printf("    For Post Tap 2 =   0  choose '0' \n");
          printf("    For Post Tap 2 =   1  choose 'G' \n");    
          printf("    For Post Tap 2 =   2  choose 'H' \n");
          printf("    For Post Tap 2 =   3  choose 'I' \n");
          printf("    For Post Tap 2 =   4  choose 'J' \n");
          printf("    For Post Tap 2 =   5  choose 'K' \n");        
          printf("    For Post Tap 2 =   6  choose 'L' \n");
          printf("    For Post Tap 2 =   7  choose 'M' \n");
          printf("    For Post Tap 2 =   8  choose 'N' \n");
          printf("    For Post Tap 2 =   9  choose 'O' \n");
          printf("    For Post Tap 2 =  10  choose 'P' \n");
          printf("    For Post Tap 2 =  11  choose 'Q' \n");        
          printf("    For Post Tap 2 =  12  choose 'R' \n");
          printf("    For Post Tap 2 =  13  choose 'S' \n");  
          printf("    For Post Tap 2 =  14  choose 'T' \n");
          printf("    For Post Tap 2 =  15  choose 'U' \n");                                                                                                    
        printf("    No Change         choose 'X' \n\n");    
        printf("    Choice :");
        
        temp = 0xffffffff;
                                                   
        rx_char = input_char(); 
        switch (rx_char)
        {
        case '0': temp =  POST_TAP2_0; break;
        case '1': temp =  POST_TAP2_15N; break;
        case '2': temp =  POST_TAP2_14N; break;
        case '3': temp =  POST_TAP2_13N; break;
        case '4': temp =  POST_TAP2_12N; break;
        case '5': temp =  POST_TAP2_11N; break;
        case '6': temp =  POST_TAP2_10N; break;
        case '7': temp =  POST_TAP2_9N; break;
        case '8': temp =  POST_TAP2_8N; break;
        case '9': temp =  POST_TAP2_7N; break;
        case 'a':
        case 'A': temp =  POST_TAP2_6N; break;
        case 'b':
        case 'B': temp =  POST_TAP2_5N; break;
        case 'c':
        case 'C': temp =  POST_TAP2_4N; break;
        case 'd':
        case 'D': temp =  POST_TAP2_3N; break;
        case 'e':
        case 'E': temp =  POST_TAP2_2N; break;      
        case 'f':
        case 'F': temp =  POST_TAP2_1N; break;   
        case 'g':
        case 'G': temp =  POST_TAP2_1; break;  
        case 'h':
        case 'H': temp =  POST_TAP2_2; break;  
        case 'i':
        case 'I': temp =  POST_TAP2_3; break;  
        case 'j':
        case 'J': temp =  POST_TAP2_4; break;   
        case 'k':
        case 'K': temp =  POST_TAP2_5; break;  
        case 'l':
        case 'L': temp =  POST_TAP2_6; break;  
        case 'm':
        case 'M': temp =  POST_TAP2_7; break;  
        case 'n':
        case 'N': temp =  POST_TAP2_8; break;   
        case 'o':
        case 'O': temp =  POST_TAP2_9; break;  
        case 'p':
        case 'P': temp =  POST_TAP2_10; break;  
        case 'q':
        case 'Q': temp =  POST_TAP2_11; break;  
        case 'r':
        case 'R': temp =  POST_TAP2_12; break;   
        case 's':
        case 'S': temp =  POST_TAP2_13; break;  
        case 't':
        case 'T': temp =  POST_TAP2_14; break;  
        case 'u':
        case 'U': temp =  POST_TAP2_15; break;           
        case 'x': 
        case 'X': temp = tx_preemp_2t[SelectedChannel];break;
        default : break;
        }
        } while (temp == 0xffffffff);     
        
        tx_preemp_2t[SelectedChannel] = temp;        
        

       do {
        printf("\n");  
        
        printf("    New Equalizer Level Channel %1d               \n",SelectedChannel);
        printf("    ============================                \n");
        printf("    For Equalizer Level  =  0 choose '0' \n");
        printf("    For Equalizer Level  =  1 choose '1' \n");
        printf("    For Equalizer Level  =  2 choose '2' \n");
        printf("    For Equalizer Level  =  3 choose '3' \n");
        printf("    For Equalizer Level  =  4 choose '4' \n");
        printf("    For Equalizer Level  =  5 choose '5' \n");
        printf("    For Equalizer Level  =  6 choose '6' \n");
        printf("    For Equalizer Level  =  7 choose '7' \n");  
        printf("    For Equalizer Level  =  8 choose '8' \n");
        printf("    For Equalizer Level  =  9 choose '9' \n");
        printf("    For Equalizer Level  = 10 choose 'A' \n");
        printf("    For Equalizer Level  = 11 choose 'B' \n");
        printf("    For Equalizer Level  = 12 choose 'C' \n");
        printf("    For Equalizer Level  = 13 choose 'D' \n");
        printf("    For Equalizer Level  = 14 choose 'E' \n");
        printf("    For Equalizer Level  = 15 choose 'F' \n");                        
        printf("    No Change                 choose 'X' \n\n");    
        printf("    Choice :");      
        
        temp = 0xffffffff;
                                             
        rx_char   = input_char();
        switch (rx_char)
        {
        case '0': temp =  EQ_CTRL_0; break;
        case '1': temp =  EQ_CTRL_1; break;
        case '2': temp =  EQ_CTRL_2; break;
        case '3': temp =  EQ_CTRL_3; break;
        case '4': temp =  EQ_CTRL_4; break;
        case '5': temp =  EQ_CTRL_5; break;
        case '6': temp =  EQ_CTRL_6; break;
        case '7': temp =  EQ_CTRL_7; break;
        case '8': temp =  EQ_CTRL_8; break;
        case '9': temp =  EQ_CTRL_9; break;
        case 'a':
        case 'A': temp =  EQ_CTRL_10; break;
        case 'b':
        case 'B': temp =  EQ_CTRL_11; break;
        case 'c':
        case 'C': temp =  EQ_CTRL_12; break;
        case 'd':
        case 'D': temp =  EQ_CTRL_13; break;
        case 'e':
        case 'E': temp =  EQ_CTRL_14; break;
        case 'f':
        case 'F': temp =  EQ_CTRL_15; break;                        
        case 'x': 
        case 'X': temp =  rx_eqctrl[SelectedChannel];break;
        default : break;
        }
        } while (temp == 0xffffffff);
 
        rx_eqctrl[SelectedChannel] = temp; 
                

       do {
        printf("\n");  
        
        printf("    New Equalizer DC Gain Channel %1d               \n",SelectedChannel);
        printf("    =============================                \n");
        printf("    For Equalizer DC Gain  =  0 choose '0' \n");
        printf("    For Equalizer DC Gain  =  1 choose '1' \n");
        printf("    For Equalizer DC Gain  =  2 choose '2' \n");     
        printf("    For Equalizer DC Gain  =  3 choose '3' \n");
        printf("    For Equalizer DC Gain  =  4 choose '4' \n");
        printf("    For Equalizer DC Gain  =  5 choose '5' \n");     
        printf("    For Equalizer DC Gain  =  6 choose '6' \n");
        printf("    For Equalizer DC Gain  =  7 choose '7' \n"); 			 
        printf("    No Change                   choose 'X' \n\n");    
        printf("    Choice :");      
        
        temp = 0xffffffff;
                                             
        rx_char   = input_char();
        switch (rx_char)
        {
        case '0': temp =  EQ_GAIN_0; break;
        case '1': temp =  EQ_GAIN_1; break;
        case '2': temp =  EQ_GAIN_2; break;
        case '3': temp =  EQ_GAIN_3; break;
        case '4': temp =  EQ_GAIN_4; break; 
        case '5': temp =  EQ_GAIN_5; break;
        case '6': temp =  EQ_GAIN_6; break;
        case '7': temp =  EQ_GAIN_7; break;             
        case 'x': 
        case 'X': temp =  rx_eqdcgain[SelectedChannel];break;
        default : break;
        }
        } while (temp == 0xffffffff);
 
        rx_eqdcgain[SelectedChannel] = temp; 
        
        // Debug Purposes
         printf("    tx_vodctrl = %d                \n",tx_vodctrl[SelectedChannel]);
         printf("    tx_preemp_0t = %d              \n",tx_preemp_0t[SelectedChannel]);
         printf("    tx_preemp_1t = %d              \n",tx_preemp_1t[SelectedChannel]);
         printf("    tx_preemp_2t = %d              \n",tx_preemp_2t[SelectedChannel]);
         printf("    rx_eqdcgain  = %d              \n",rx_eqdcgain[SelectedChannel] );
         printf("    rx_eqctrl  = %d              \n",rx_eqctrl[SelectedChannel] );

  
		   write_reconfiguration(SelectedChannel+4,VOD,tx_vodctrl[SelectedChannel]);
			write_reconfiguration(SelectedChannel+4,PRE_TAP,tx_preemp_0t[SelectedChannel]);
			write_reconfiguration(SelectedChannel+4,POST_TAP1,tx_preemp_1t[SelectedChannel]);	  
			write_reconfiguration(SelectedChannel+4,POST_TAP2,tx_preemp_2t[SelectedChannel]);	
			write_reconfiguration(SelectedChannel,EQ_GAIN,rx_eqdcgain[SelectedChannel]);	 
			write_reconfiguration(SelectedChannel,EQ_CTRL,rx_eqctrl[SelectedChannel]);	 
		      
                      
				printf("\nTransceiver PMA parameters of Channel %1d updated\n\n",SelectedChannel);
				
				break; //case '4'
        
			case '5': /* Insert Biterrors */
     
        printf("5. Insert Biterror on Channel %1d\n\n\n",SelectedChannel);


        Control_Reg[0] = Control_Reg[0] | (0x4000);
        IOWR_ALTERA_AVALON_PIO_DATA(CONTROL_REG_BASE,Control_Reg[0]);
        
        Control_Reg[0] = Control_Reg[0] & (0xBFFF);
        IOWR_ALTERA_AVALON_PIO_DATA(CONTROL_REG_BASE,Control_Reg[0]);
   
         
		break;
	
			case '6': /* Reset ErrorCounter */
        
	      printf("6. Reset ErrorCounter on Selected Channel %1d\n\n\n",SelectedChannel);
   
        
        Control_Reg[0] = Control_Reg[0] | (0x2000);
        IOWR_ALTERA_AVALON_PIO_DATA(CONTROL_REG_BASE,Control_Reg[0]);
        
        Control_Reg[0] = Control_Reg[0] & (0xDFFF);
        IOWR_ALTERA_AVALON_PIO_DATA(CONTROL_REG_BASE,Control_Reg[0]);
       
        usleep(1000000);     
            
       break;
	
      case '7': /* Show Status */ 
            
       break;      

//      case '8': /*Toggle Serial Loopback on selected Channel and reset */
//         
//
//        	if (Loopback[SelectedChannel] == 0)
//        	       Loopback[SelectedChannel] = 1;
//        	else
//        	       Loopback[SelectedChannel] = 0; 
//        
//          Loopback_All = (Loopback_All & (0xFFFE)) | (Loopback[0]);
//          Loopback_All = (Loopback_All & (0xFFFD)) | (Loopback[1] << 1);
//          Loopback_All = (Loopback_All & (0xFFFB)) | (Loopback[2] << 2);
//          Loopback_All = (Loopback_All & (0xFFF7)) | (Loopback[3] << 3); 			
//			
//			 write_Loopback(Loopback_All);
//          
//          Control_Reg[0] = Control_Reg[0] | (0x8000);
//          IOWR_ALTERA_AVALON_PIO_DATA(CONTROL_REG_BASE,Control_Reg[0]);
//          usleep(10);
//          
//          Control_Reg[0] = Control_Reg[0] & (0x7FFF);
//          IOWR_ALTERA_AVALON_PIO_DATA(CONTROL_REG_BASE,Control_Reg[0]);
//          
//		  usleep(1000000); 
//			
//        Control_Reg[0] = Control_Reg[0] | (0x2000);
//        IOWR_ALTERA_AVALON_PIO_DATA(CONTROL_REG_BASE,Control_Reg[0]);
//        
//        Control_Reg[0] = Control_Reg[0] & (0xDFFF);
//        IOWR_ALTERA_AVALON_PIO_DATA(CONTROL_REG_BASE,Control_Reg[0]);
//       
//          
//		  
//        usleep(1000000);
//        
//        printf("Toggled Serial Loopback on selected channel and reset .\n\n");
//        break;
     

      
      case 'a':
      case 'A': /*Select Prbs Pattern*/

      printf("A. Select Prsb Pattern on Selected Channel %1d and Reset\n",SelectedChannel);
      printf("    Current Prbs Pattern Selected   : ");
      switch (PrbsSelect[SelectedChannel])
       {
       case  0  : printf(" PRBS-7\n"); break;
       case  1  : printf(" PRBS-23\n"); break;
       case  2  : printf(" PRBS-31\n"); break;
       case  3  : printf(" PRBS-15\n"); break;
       default : break;
       }  
     
     
        do {
        printf("\n");  
        
        printf("    New Prbs Pattern on Channel %1d               \n",SelectedChannel);
        printf("    ============================                \n");
        printf("    For PRBS-7   choose '0' \n");
        printf("    For PRBS-23  choose '1' \n");
        printf("    For PRBS-31  choose '2' \n");
        printf("    For PRBS-15  choose '3' \n");                             
        printf("    No Change    choose 'X' \n\n");    
        printf("    Choice :");             
        
        temp = 0xffffffff;
                                             
        rx_char   = input_char();
        switch (rx_char)
        {
        case '0': temp =  PRBS_7; break;
        case '1': temp =  PRBS_23; break;
        case '2': temp =  PRBS_31; break;
        case '3': temp =  PRBS_15; break;                        
        case 'x': 
        case 'X': temp =  PrbsSelect[SelectedChannel];break;
        default : break;
        }
        } while (temp == 0xffffffff);
 
        PrbsSelect[SelectedChannel] = temp; 
       
          Control2_Reg[0] = Control2_Reg[0] | (0x1000); /* Latch PrbsPattern Assert */
          Control2_Reg[0] = (Control2_Reg[0] & (0xFCFF)) | (temp << 8); 
          IOWR_ALTERA_AVALON_PIO_DATA(CONTROL2_REG_BASE,Control2_Reg[0]);
          
          Control2_Reg[0] = Control2_Reg[0] & (0xEFFF); /* Latch PrbsPattern Deassert */
          IOWR_ALTERA_AVALON_PIO_DATA(CONTROL2_REG_BASE,Control2_Reg[0]);
          
          Control_Reg[0] = Control_Reg[0] | (0x8000);
          IOWR_ALTERA_AVALON_PIO_DATA(CONTROL_REG_BASE,Control_Reg[0]);
          usleep(10);
          
          Control_Reg[0] = Control_Reg[0] & (0x7FFF);
          IOWR_ALTERA_AVALON_PIO_DATA(CONTROL_REG_BASE,Control_Reg[0]);
          
      usleep(1000000); 
      break; //case 'A'
      
     case 'b':
     case 'B': 
        Show_ErrorCount = 1;
        break;
     
     case 'c': //continuously update BER and errorcount every BERInterval seconds
     case 'C': 

        printf("\n");  
        printf("Ch %1d BER + ErrorCount every %d seconds              \n",SelectedChannel,BERInterval);
        printf("Press any key + Enter to stop the loop\n");
        printf("=========================================               \n");
        
        temp = 0;
        
        Reg_data = IORD_ALTERA_AVALON_JTAG_UART_DATA(ALT_STDIN_BASE);
        // Shift rigth of 16 to get access to the number of characters remaining in the read fifo
        Ravail = Reg_data >> 16;
        
        //printf("\nRavail %d :\n",Ravail);
        
        do
          {
        
        // Reset ErrorCount on Selected Channel

            Control_Reg[0] = Control_Reg[0] | (0x2000);
            IOWR_ALTERA_AVALON_PIO_DATA(CONTROL_REG_BASE,Control_Reg[0]);
            
            Control_Reg[0] = Control_Reg[0] & (0xDFFF);
            IOWR_ALTERA_AVALON_PIO_DATA(CONTROL_REG_BASE,Control_Reg[0]);

            
        TimeInterval = BERInterval * 1000;
        

			do
			{
			Counter_1ms_Reg[0]     = IORD_ALTERA_AVALON_PIO_DATA(COUNTER_1MS_REG_BASE);
			} while (Counter_1ms_Reg[0] < TimeInterval);
   

                     
             
    
        
            //Read ErrorCount registers
            
            Counter_1ms_Reg[0]     = IORD_ALTERA_AVALON_PIO_DATA(COUNTER_1MS_REG_BASE);
           
			  ErrorCount_Reg_L[0]	 = IORD_ALTERA_AVALON_PIO_DATA(ERRORCOUNT_L_0_BASE);
			  ErrorCount_Reg_H[0]	 = IORD_ALTERA_AVALON_PIO_DATA(ERRORCOUNT_H_0_BASE);
			  ErrorCount_Reg_L[1]	 = IORD_ALTERA_AVALON_PIO_DATA(ERRORCOUNT_L_1_BASE);
			  ErrorCount_Reg_H[1]	 = IORD_ALTERA_AVALON_PIO_DATA(ERRORCOUNT_H_1_BASE);
			  ErrorCount_Reg_L[2]	 = IORD_ALTERA_AVALON_PIO_DATA(ERRORCOUNT_L_2_BASE);
			  ErrorCount_Reg_H[2]	 = IORD_ALTERA_AVALON_PIO_DATA(ERRORCOUNT_H_2_BASE);
			  ErrorCount_Reg_L[3]	 = IORD_ALTERA_AVALON_PIO_DATA(ERRORCOUNT_L_3_BASE);
			  ErrorCount_Reg_H[3]	 = IORD_ALTERA_AVALON_PIO_DATA(ERRORCOUNT_H_3_BASE);
            
                     
   
           for (i = 0; i < NUMBER_OF_LINKS ; i++)
              {
              ErrorCount[i] = ((float) (ErrorCount_Reg_H[i]) *  MULTIPLIER * MULTIPLIER) + (float) (ErrorCount_Reg_L[i]);
              }

            Totalbits_tx[0] = (float) (Counter_1ms_Reg[0]) * (float) (Bitrate[datarate_tx]*1000);
            Totalbits_tx[1] = (float) (Counter_1ms_Reg[0]) * (float) (Bitrate[datarate_tx]*1000);
            Totalbits_tx[2] = (float) (Counter_1ms_Reg[0]) * (float) (Bitrate[datarate_tx]*1000);
            Totalbits_tx[3] = (float) (Counter_1ms_Reg[0]) * (float) (Bitrate[datarate_tx]*1000);

  
            for (i = 0; i < NUMBER_OF_LINKS ; i++)
            {
                if (ErrorCount[i] > 0)
                  BER[i] = ((ErrorCount[i])) / Totalbits_tx[i];
                else
                  BER[i] = 0; 
            } /* For Loop */   
            

            if (Powerdown[SelectedChannel] == 1)
              printf("Ch%2d Powered down\n",SelectedChannel);        
            else if (Locked[SelectedChannel] == 1)
                {
                if (ErrorCount[SelectedChannel] > 0)
                        printf("Ch %1d : BER Estimate : %e    ErrorCount  : %e \n",SelectedChannel,BER[SelectedChannel],ErrorCount[SelectedChannel]);
                    else
                        printf("Ch %1d : BER          : 0\n",SelectedChannel);
                }
            else
              printf("No Data Lock on Ch%2d \n",SelectedChannel);
         
                
           
        temp = temp + 1;
                                           
 
        //} while (temp <= 1000); // loop a 1000 times
        Reg_data = IORD_ALTERA_AVALON_JTAG_UART_DATA(ALT_STDIN_BASE);
        Ravail = Reg_data >> 16;
        } while(!Ravail);
        rx_char   = input_char();
        
        printf("\n Stopped\n");
        
        break;

      case 'd':
      case 'D': /* New BER Time Interval */
        
        printf("D. Provide BER Time Interval\n");
        printf("   Insert a Hexadecimal Number from 01 to FF\n");
        printf("   Current BER Time Interval   : %2d seconds\n", BERInterval);
       
        temp = 0xffffffff;
           do {
            printf("   New Interval (Upper Nibble) (0-F)        :");
            rx_char   = input_char();
        
            switch (rx_char)
            {
            case '0': temp = 0; break;
            case '1': temp = 1; break;
            case '2': temp = 2; break;
            case '3': temp = 3; break;
            case '4': temp = 4; break;
            case '5': temp = 5; break;
            case '6': temp = 6; break;
            case '7': temp = 7; break;
            case '8': temp = 8; break;
            case '9': temp = 9; break;
            case 'A': 
            case 'a': temp = 0xa; break;
            case 'B': 
            case 'b': temp = 0xb; break;
            case 'C': 
            case 'c': temp = 0xc; break;
            case 'D': 
            case 'd': temp = 0xd; break;
            case 'E': 
            case 'e': temp = 0xe; break;
            case 'F': 
            case 'f': temp = 0xf; break;
            default : break;
            }
          } while (temp == 0xffffffff);
          
          High_nibble = temp;
          
          temp = 0xffffffff;
           do {
            printf("\n   New Interval (Lower Nibble) (0-F)        :");
            rx_char   = input_char();
        
            switch (rx_char)
            {
            case '0': temp = 0; break;
            case '1': temp = 1; break;
            case '2': temp = 2; break;
            case '3': temp = 3; break;
            case '4': temp = 4; break;
            case '5': temp = 5; break;
            case '6': temp = 6; break;
            case '7': temp = 7; break;
            case '8': temp = 8; break;
            case '9': temp = 9; break;
            case 'A': 
            case 'a': temp = 0xa; break;
            case 'B': 
            case 'b': temp = 0xb; break;
            case 'C': 
            case 'c': temp = 0xc; break;
            case 'D': 
            case 'd': temp = 0xd; break;
            case 'E': 
            case 'e': temp = 0xe; break;
            case 'F': 
            case 'f': temp = 0xf; break;
            default : break;
            }
          } while (temp == 0xffffffff);
          
          Low_nibble = temp;  
    
          BERInterval = (High_nibble << 4) | Low_nibble;
         
            
          printf("    New Time interval is %2d seconds:\n",BERInterval);               
            
       break; /* Case D */
       
      case 'e': /* Show Transceiver PMA Settings on all channels */
      case 'E':
   	  printf("\n\n");
    	  printf("Channel         :|");
    	      for (i = 0; i < NUMBER_OF_LINKS ; i++)
    	      { 
    	        printf("%4X|",i);
    	      }
    	      printf("\n");
    	      printf("                 |");
    	      for (i = 0; i < NUMBER_OF_LINKS ; i++)
    	      { 
    	      printf("----|");
    	      }
    	      printf("\n");
    	   
 
    	      printf("VOD(mV)         :|"); 
    	      for (i = 0; i < NUMBER_OF_LINKS ; i++)
    	      { 
    	          switch (tx_vodctrl[i])
    	           {
    	           case  VOD_200MV  : printf(" 200|"); break;
    	           case  VOD_400MV  : printf(" 400|"); break;
    	           case  VOD_600MV  : printf(" 600|"); break;
    	           case  VOD_700MV  : printf(" 700|"); break;          
    	           case  VOD_800MV  : printf(" 800|"); break;
    	           case  VOD_900MV  : printf(" 900|"); break;          
    	           case  VOD_1000MV : printf("1000|"); break;
    	           case  VOD_1200MV : printf("1200|"); break;                                                              
    	           default : break;
    	           }    	    	  
    	      }
    	      printf("\n");
    	      
    	      printf("Pre Tap Level   :|"); 
    	      for (i = 0; i < NUMBER_OF_LINKS ; i++)
    	      { 
    	           switch (tx_preemp_0t[i])
    	           {
    	           case  PRE_TAP_15N : printf(" -15|"); break;
    	           case  PRE_TAP_14N : printf(" -14|"); break;
    	           case  PRE_TAP_13N : printf(" -13|"); break;
    	           case  PRE_TAP_12N : printf(" -12|"); break;
    	           case  PRE_TAP_11N : printf(" -11|"); break;
    	           case  PRE_TAP_10N : printf(" -10|"); break;
    	           case  PRE_TAP_9N  : printf("  -9|"); break;  
    	           case  PRE_TAP_8N  : printf("  -8|"); break;              
    	           case  PRE_TAP_7N  : printf("  -7|"); break;
    	           case  PRE_TAP_6N  : printf("  -6|"); break;
    	           case  PRE_TAP_5N  : printf("  -5|"); break;
    	           case  PRE_TAP_4N  : printf("  -4|"); break;
    	           case  PRE_TAP_3N  : printf("  -3|"); break;
    	           case  PRE_TAP_2N  : printf("  -2|"); break;
    	           case  PRE_TAP_1N  : printf("  -1|"); break;
    	           case  PRE_TAP_0   : printf("   0|"); break;
    	           case  PRE_TAP_1   : printf("   1|"); break;
    	           case  PRE_TAP_2   : printf("   2|"); break;
    	           case  PRE_TAP_3   : printf("   3|"); break;
    	           case  PRE_TAP_4   : printf("   4|"); break;
    	           case  PRE_TAP_5   : printf("   5|"); break;
    	           case  PRE_TAP_6   : printf("   6|"); break;
    	           case  PRE_TAP_7   : printf("   7|"); break; 
    	           case  PRE_TAP_8   : printf("   8|"); break;
    	           case  PRE_TAP_9   : printf("   9|"); break;
    	           case  PRE_TAP_10  : printf("  10|"); break;
    	           case  PRE_TAP_11  : printf("  11|"); break;
    	           case  PRE_TAP_12  : printf("  12|"); break;
    	           case  PRE_TAP_13  : printf("  13|"); break;
    	           case  PRE_TAP_14  : printf("  14|"); break;
    	           case  PRE_TAP_15  : printf("  15|"); break;               
    	           default : break;
    	           } 	    	  
    	      }
    	      printf("\n");
 
    	      printf("Post Tap 1 Level:|"); 
    	      for (i = 0; i < NUMBER_OF_LINKS ; i++)
    	      { 
    	           switch (tx_preemp_1t[i])
    	           {
    	           case  POST_TAP1_0  : printf("   0|"); break;
    	           case  POST_TAP1_1  : printf("   1|"); break;    
    	           case  POST_TAP1_2  : printf("   2|"); break;
    	           case  POST_TAP1_3  : printf("   3|"); break;   
    	           case  POST_TAP1_4  : printf("   4|"); break;
    	           case  POST_TAP1_5  : printf("   5|"); break;   
    	           case  POST_TAP1_6  : printf("   6|"); break;
    	           case  POST_TAP1_7  : printf("   7|"); break;   
    	           case  POST_TAP1_8  : printf("   8|"); break;
    	           case  POST_TAP1_9  : printf("   9|"); break;   
    	           case  POST_TAP1_10 : printf("  10|"); break;
    	           case  POST_TAP1_11 : printf("  11|"); break;   
    	           case  POST_TAP1_12 : printf("  12|"); break;     
    	           case  POST_TAP1_13 : printf("  13|"); break;   
    	           case  POST_TAP1_14 : printf("  14|"); break;
    	           case  POST_TAP1_15 : printf("  15|"); break;   
    	           case  POST_TAP1_16 : printf("  16|"); break;
    	           case  POST_TAP1_17 : printf("  17|"); break;   
    	           case  POST_TAP1_18 : printf("  18|"); break;
    	           case  POST_TAP1_19 : printf("  19|"); break;   
    	           case  POST_TAP1_20 : printf("  20|"); break;
    	           case  POST_TAP1_21 : printf("  21|"); break;   
    	           case  POST_TAP1_22 : printf("  22|"); break; 
    	           case  POST_TAP1_23 : printf("  23|"); break;   
    	           case  POST_TAP1_24 : printf("  24|"); break;
    	           case  POST_TAP1_25 : printf("  25|"); break;   
    	           case  POST_TAP1_26 : printf("  26|"); break;
    	           case  POST_TAP1_27 : printf("  27|"); break;   
    	           case  POST_TAP1_28 : printf("  28|"); break;
    	           case  POST_TAP1_29 : printf("  29|"); break;   
    	           case  POST_TAP1_30 : printf("  30|"); break;
    	           case  POST_TAP1_31 : printf("  31|"); break;             
    	           default : break;
    	           }    	  
    	      }
    	      printf("\n");

    	      printf("Post Tap 2 Level:|"); 
    	      for (i = 0; i < NUMBER_OF_LINKS ; i++)
    	      { 

    	           switch (tx_preemp_2t[i])
    	           {
    	           case  POST_TAP2_15N : printf(" -15|"); break;
    	           case  POST_TAP2_14N : printf(" -14|"); break;
    	           case  POST_TAP2_13N : printf(" -13|"); break;
    	           case  POST_TAP2_12N : printf(" -12|"); break;
    	           case  POST_TAP2_11N : printf(" -11|"); break;
    	           case  POST_TAP2_10N : printf(" -10|"); break;
    	           case  POST_TAP2_9N  : printf("  -9|"); break;  
    	           case  POST_TAP2_8N  : printf("  -8|"); break;              
    	           case  POST_TAP2_7N  : printf("  -7|"); break;
    	           case  POST_TAP2_6N  : printf("  -6|"); break;
    	           case  POST_TAP2_5N  : printf("  -5|"); break;
    	           case  POST_TAP2_4N  : printf("  -4|"); break;
    	           case  POST_TAP2_3N  : printf("  -3|"); break;
    	           case  POST_TAP2_2N  : printf("  -2|"); break;
    	           case  POST_TAP2_1N  : printf("  -1|"); break;
    	           case  POST_TAP2_0   : printf("   0|"); break;
    	           case  POST_TAP2_1   : printf("   1|"); break;
    	           case  POST_TAP2_2   : printf("   2|"); break;
    	           case  POST_TAP2_3   : printf("   3|"); break;
    	           case  POST_TAP2_4   : printf("   4|"); break;
    	           case  POST_TAP2_5   : printf("   5|"); break;
    	           case  POST_TAP2_6   : printf("   6|"); break;
    	           case  POST_TAP2_7   : printf("   7|"); break; 
    	           case  POST_TAP2_8   : printf("   8|"); break;
    	           case  POST_TAP2_9   : printf("   9|"); break;
    	           case  POST_TAP2_10  : printf("  10|"); break;
    	           case  POST_TAP2_11  : printf("  11|"); break;
    	           case  POST_TAP2_12  : printf("  12|"); break;
    	           case  POST_TAP2_13  : printf("  13|"); break;
    	           case  POST_TAP2_14  : printf("  14|"); break;
    	                                                                                                                                                                                                           
    	           default : break;
    	           }   	  
    	      }
    	      printf("\n");    	      
 
    	      printf("Eq Gain Level   :|"); 
    	      for (i = 0; i < NUMBER_OF_LINKS ; i++)
    	      { 

    	           switch (rx_eqdcgain[i])
    	           {
    	           case  EQ_GAIN_0    : printf("   0|"); break;
    	           case  EQ_GAIN_1    : printf("   1|"); break;
    	           case  EQ_GAIN_2    : printf("   2|"); break;   
    	           case  EQ_GAIN_3    : printf("   3|"); break;
    	           case  EQ_GAIN_4    : printf("   4|"); break;
    	           case  EQ_GAIN_5    : printf("   5|"); break;   
    	           case  EQ_GAIN_6    : printf("   6|"); break;
    	           case  EQ_GAIN_7    : printf("   7|"); break;						  
    	           default : break;
    	           } 	  
    	      }
    	      printf("\n");    

    	      printf("Eq Control Level:|"); 
    	      for (i = 0; i < NUMBER_OF_LINKS ; i++)
    	      { 

    	           switch (rx_eqctrl[i])
    	           {
    	           case  EQ_CTRL_0    : printf("   0|"); break;
    	           case  EQ_CTRL_1    : printf("   1|"); break;
    	           case  EQ_CTRL_2    : printf("   2|"); break;
    	           case  EQ_CTRL_3    : printf("   3|"); break;
    	           case  EQ_CTRL_4    : printf("   4|"); break;
    	           case  EQ_CTRL_5    : printf("   5|"); break;
    	           case  EQ_CTRL_6    : printf("   6|"); break;
    	           case  EQ_CTRL_7    : printf("   7|"); break;
    	           case  EQ_CTRL_8    : printf("   8|"); break;
    	           case  EQ_CTRL_9    : printf("   9|"); break;
    	           case  EQ_CTRL_10   : printf("  10|"); break;
    	           case  EQ_CTRL_11   : printf("  11|"); break;
    	           case  EQ_CTRL_12   : printf("  12|"); break;
    	           case  EQ_CTRL_13   : printf("  13|"); break;
    	           case  EQ_CTRL_14   : printf("  14|"); break;   
    	           case  EQ_CTRL_15   : printf("  15|"); break;                                                                                                                                                                                                                                                                                    
    	           default : break;
    	           }	  
    	      }
    	      printf("\n"); 

    	  
    	  break; // Case E

  		case 'f':
        case 'F': /*Toggle Reverse Parallel Loopback on channel (if Available) */
        

  		 if (Rev_Parallel_Available[SelectedChannel] == 0)
  			 {
  			 printf("\n\nChannel does not support Reverse Parallel Loopback.\n");
  			 break;
  			 }
  		 else
			{
             if (Rev_Parallel[SelectedChannel] == 0)
          	         Rev_Parallel[SelectedChannel] = 1;
          	 else
          	        Rev_Parallel[SelectedChannel] = 0; 
          	          
               }
            Control_Reg[0] = (Control_Reg[0] & (0xFFEF)) | (Rev_Parallel[0] << 4);
            Control_Reg[0] = (Control_Reg[0] & (0xFFDF)) | (Rev_Parallel[1] << 5);
            Control_Reg[0] = (Control_Reg[0] & (0xFFBF)) | (Rev_Parallel[2] << 6);
            Control_Reg[0] = (Control_Reg[0] & (0xFF7F)) | (Rev_Parallel[3] << 7); 
            IOWR_ALTERA_AVALON_PIO_DATA(CONTROL_REG_BASE,Control_Reg[0]);
            
            Control_Reg[0] = Control_Reg[0] | (0x8000);
            IOWR_ALTERA_AVALON_PIO_DATA(CONTROL_REG_BASE,Control_Reg[0]);
            usleep(10);
            
            Control_Reg[0] = Control_Reg[0] & (0x7FFF);
            IOWR_ALTERA_AVALON_PIO_DATA(CONTROL_REG_BASE,Control_Reg[0]);
            
            
                
          usleep(1000000);
          
          printf("Toggled Reverse Parallel Loopback And Reset .\n\n");
          break;

					
//	case 'r':
//	case 'R' : //Change Datarate on Rx
//		
//	   printf("\nCurrent selected datarate on Rx is ");
//			  switch (datarate_rx)
//			  {
//			  case  RATE_5400MBPS    : printf("5400 Mbps from 108 Mhz Reference Clock\n"); break;
//			  case  RATE_2700MBPS    : printf("2700 Mbps from 108 Mhz Reference Clock\n"); break;
//			  case  RATE_1620MBPS    : printf("1620 Mbps from 162 Mhz Reference Clock\n"); break;
//			  default : break;
//			  }	
//
//       do {
//        printf("\n");  
//        
//        printf("    New Rx Datarate             \n");
//        printf("    ============================                \n");
//        printf("    For 5400 Mbps from 108 Mhz Reference Clock choose '0' \n");
//        printf("    For 2700 Mbps from 108 Mhz Reference Clock choose '1' \n");
//        printf("    For 1620 Mbps from 162 Mhz Reference Clock choose '2' \n");                           
//        printf("    No Change    choose 'X' \n\n");    
//        printf("    Choice :");             
//        
//        temp = 0xffffffff;
//                                             
//        rx_char   = input_char();
//        switch (rx_char)
//        {
//        case '0': temp =  RATE_5400MBPS; break;
//        case '1': temp =  RATE_2700MBPS; break;
//        case '2': temp =  RATE_1620MBPS; break;                     
//        case 'x': 
//        case 'X': temp =  datarate_rx;break;
//        default : break;
//        }
//        } while (temp == 0xffffffff);
// 
//        datarate_rx = temp; 			  
//		
//		hw_dprio = 1;
//		Select_tx_rxn = 0;
//		  
//		// Set DPRIO through hardware
//		Control_Reg[0] = (Control_Reg[0] & (0xFFFE)) | (hw_dprio);
//		
//		// Set Datarate 
//		Control_Reg[0] = (Control_Reg[0] & (0xF9FF)) | (datarate_rx << 9);
//
//		// Select Rx to reconfig (Control_reg(11)) 
//		Control_Reg[0] = (Control_Reg[0] & (0xF7FF)) | (Select_tx_rxn << 11);
//		  
//		IOWR_ALTERA_AVALON_PIO_DATA(CONTROL_REG_BASE,Control_Reg[0]);
//		
//		// Pulse Do_Reconfig
//		Control_Reg[0] = Control_Reg[0] | (0x1000);
//		IOWR_ALTERA_AVALON_PIO_DATA(CONTROL_REG_BASE,Control_Reg[0]);
//		usleep(10);
//		
//		Control_Reg[0] = Control_Reg[0] & (0xEFFF);
//		IOWR_ALTERA_AVALON_PIO_DATA(CONTROL_REG_BASE,Control_Reg[0]);		  
//
//     //Wait for reconfiguration to finish
//	  do
//     {
//     Channel_Reg[0]      = IORD_ALTERA_AVALON_PIO_DATA(CHANNEL0_REG_BASE);
//	  Reconfig_end = 0x0001 & (Channel_Reg[0] >> 13); 
//	  } while (Reconfig_end == 0);		
//	
//	 //Give reconfiguration control back to software
//	 	hw_dprio = 0;
//		
//		// Set DPRIO through hardware
//		Control_Reg[0] = (Control_Reg[0] & (0xFFFE)) | (hw_dprio);
//		
//		IOWR_ALTERA_AVALON_PIO_DATA(CONTROL_REG_BASE,Control_Reg[0]);
//	  
//		Control_Reg[0] = Control_Reg[0] | (0x8000);
//		IOWR_ALTERA_AVALON_PIO_DATA(CONTROL_REG_BASE,Control_Reg[0]);
//		usleep(10);
//		
//		Control_Reg[0] = Control_Reg[0] & (0x7FFF);
//		IOWR_ALTERA_AVALON_PIO_DATA(CONTROL_REG_BASE,Control_Reg[0]);
//	  
//		usleep(1000000);		  
//		  
//		printf("\nChanged Datarate on Rx side");	
//		
//		show_coreclks = 0; // don't show the coreclk's yet as it takes time to measure them.  
//		
//      break;
//
//	case 't':
//	case 'T' : //Change Datarate on Tx
//		
//	   printf("\nCurrent selected datarate on Tx is ");
//			  switch (datarate_tx)
//			  {
//			  case  RATE_5400MBPS    : printf("5400 Mbps from 108 Mhz Reference Clock\n"); break;
//			  case  RATE_2700MBPS    : printf("2700 Mbps from 108 Mhz Reference Clock\n"); break;
//			  case  RATE_1620MBPS    : printf("1620 Mbps from 162 Mhz Reference Clock\n"); break;
//			  default : break;
//			  }	
//
//       do {
//        printf("\n");  
//        
//        printf("    New Tx Datarate             \n");
//        printf("    ============================                \n");
//        printf("    For 5400 Mbps from 108 Mhz Reference Clock choose '0' \n");
//        printf("    For 2700 Mbps from 108 Mhz Reference Clock choose '1' \n");
//        printf("    For 1620 Mbps from 162 Mhz Reference Clock choose '2' \n");                           
//        printf("    No Change    choose 'X' \n\n");    
//        printf("    Choice :");             
//        
//        temp = 0xffffffff;
//                                             
//        rx_char   = input_char();
//        switch (rx_char)
//        {
//        case '0': temp =  RATE_5400MBPS; break;
//        case '1': temp =  RATE_2700MBPS; break;
//        case '2': temp =  RATE_1620MBPS; break;                     
//        case 'x': 
//        case 'X': temp =  datarate_tx;break;
//        default : break;
//        }
//        } while (temp == 0xffffffff);
// 
//        datarate_tx = temp; 			  
//		
//		hw_dprio = 1;
//		Select_tx_rxn = 1;
//		  
//		// Set DPRIO through hardware
//		Control_Reg[0] = (Control_Reg[0] & (0xFFFE)) | (hw_dprio);
//		
//		// Set Datarate 
//		Control_Reg[0] = (Control_Reg[0] & (0xF9FF)) | (datarate_tx << 9);
//
//		// Select Tx to reconfig (Control_reg(11)) 
//		Control_Reg[0] = (Control_Reg[0] & (0xF7FF)) | (Select_tx_rxn << 11);
//		  
//		IOWR_ALTERA_AVALON_PIO_DATA(CONTROL_REG_BASE,Control_Reg[0]);
//		
//		// Pulse Do_Reconfig
//		Control_Reg[0] = Control_Reg[0] | (0x1000);
//		IOWR_ALTERA_AVALON_PIO_DATA(CONTROL_REG_BASE,Control_Reg[0]);
//		usleep(10);
//		
//		Control_Reg[0] = Control_Reg[0] & (0xEFFF);
//		IOWR_ALTERA_AVALON_PIO_DATA(CONTROL_REG_BASE,Control_Reg[0]);		  
//
//     //Wait for reconfiguration to finish
//	  do
//     {
//     Channel_Reg[0]      = IORD_ALTERA_AVALON_PIO_DATA(CHANNEL0_REG_BASE);
//	  Reconfig_end = 0x0001 & (Channel_Reg[0] >> 13); 
//	  } while (Reconfig_end == 0);		
//	
//	 //Give reconfiguration control back to software
//	 	hw_dprio = 0;
//		
//		// Set DPRIO through hardware
//		Control_Reg[0] = (Control_Reg[0] & (0xFFFE)) | (hw_dprio);
//		
//		IOWR_ALTERA_AVALON_PIO_DATA(CONTROL_REG_BASE,Control_Reg[0]);
//	  
//		Control_Reg[0] = Control_Reg[0] | (0x8000);
//		IOWR_ALTERA_AVALON_PIO_DATA(CONTROL_REG_BASE,Control_Reg[0]);
//		usleep(10);
//		
//		Control_Reg[0] = Control_Reg[0] & (0x7FFF);
//		IOWR_ALTERA_AVALON_PIO_DATA(CONTROL_REG_BASE,Control_Reg[0]);
//	  
//		usleep(1000000);		  
//		  
//		printf("\nChanged Datarate on Tx side");	
//		
//		show_coreclks = 0; // don't show the coreclk's yet as it takes time to measure them.  
//		
//      break;
		
	case 't':
	case 'T' : //Change Datarate on Tx using Software
		
	   printf("\nCurrent selected datarate on Tx is ");
			  switch (datarate_tx)
			  {
			  case  RATE_5400MBPS    : printf("5400 Mbps from 135 Mhz Reference Clock\n"); break;
			  case  RATE_2700MBPS    : printf("2700 Mbps from 135 Mhz Reference Clock\n"); break;
			  case  RATE_1620MBPS    : printf("1620 Mbps from 135 Mhz Reference Clock\n"); break;
			  case  RATE_2970MBPS    : printf("2970 Mbps from 297 Mhz Reference Clock\n"); break;				  
			  default : break;
			  }	

       do {
        printf("\n");  
        
        printf("    New Tx Datarate             \n");
        printf("    ============================                \n");
        printf("    For 5400 Mbps from 135 Mhz Reference Clock choose '0' \n");
        printf("    For 2700 Mbps from 135 Mhz Reference Clock choose '1' \n");
        printf("    For 1620 Mbps from 135 Mhz Reference Clock choose '2' \n");  
        printf("    For 2970 Mbps from 297 Mhz Reference Clock choose '3' \n"); 			 
        printf("    No Change    choose 'X' \n\n");    
        printf("    Choice :");             
        
        temp = 0xffffffff;
                                             
        rx_char   = input_char();
        switch (rx_char)
        {
        case '0': temp =  RATE_5400MBPS; Refclk_select_tx = TX_5400MBPS_REFCLK; 
//						tx_reconfig_data[0] = TX_5400MBPS_4 ;
//						tx_reconfig_data[1] = TX_5400MBPS_6 ;
//						tx_reconfig_data[2] = TX_5400MBPS_8 ;
						tx_reconfig_data[3] = PLL_5400MBPS_4 ;
						tx_reconfig_data[4] = PLL_5400MBPS_6 ;
						tx_reconfig_data[5] = PLL_5400MBPS_7 ;
						break;
        case '1': temp =  RATE_2700MBPS; Refclk_select_tx = TX_2700MBPS_REFCLK; 
//						tx_reconfig_data[0] = TX_2700MBPS_4 ;
//						tx_reconfig_data[1] = TX_2700MBPS_6 ;
//						tx_reconfig_data[2] = TX_2700MBPS_8 ;
						tx_reconfig_data[3] = PLL_2700MBPS_4 ;
						tx_reconfig_data[4] = PLL_2700MBPS_6 ;
						tx_reconfig_data[5] = PLL_2700MBPS_7 ;	  
						break;			  
        case '2': temp =  RATE_1620MBPS; Refclk_select_tx = TX_1620MBPS_REFCLK;
//						tx_reconfig_data[0] = TX_1620MBPS_4 ;
//						tx_reconfig_data[1] = TX_1620MBPS_6 ;
//						tx_reconfig_data[2] = TX_1620MBPS_8 ;
						tx_reconfig_data[3] = PLL_1620MBPS_4 ;
						tx_reconfig_data[4] = PLL_1620MBPS_6 ;
						tx_reconfig_data[5] = PLL_1620MBPS_7 ;	 		  
						break;			  
        case '3': temp =  RATE_2970MBPS; Refclk_select_tx = TX_2970MBPS_REFCLK; 
//						tx_reconfig_data[0] = TX_2970MBPS_4 ;
//						tx_reconfig_data[1] = TX_2970MBPS_6 ;
//						tx_reconfig_data[2] = TX_2970MBPS_8 ;
						tx_reconfig_data[3] = PLL_2970MBPS_4 ;
						tx_reconfig_data[4] = PLL_2970MBPS_6 ;
						tx_reconfig_data[5] = PLL_2970MBPS_7 ;	 		  
						break;			  
        case 'x':  
        case 'X': temp =  datarate_tx;break;
        default : break;
        }
        } while (temp == 0xffffffff);
 
        datarate_tx = temp; 			  
		
		// Set Datarate 
		Control_Reg[0] = (Control_Reg[0] & (0xF9FF)) | (datarate_tx << 9);

		IOWR_ALTERA_AVALON_PIO_DATA(CONTROL_REG_BASE,Control_Reg[0]);

//		  for (i = (NUMBER_OF_LINKS); i < (2*NUMBER_OF_LINKS) ; i++)
//		  {
//		  if (DEBUG == 1)
//				printf("Reconfigure logical channel with new Transmit channel info %d\n",i); // This will reconfigure the tx channel 4 .. 7
//		  write_reconfiguration_direct_register_mode_1(i,0x0000,tx_reconfig_data[0]); // Line 4 (offset 0)
//		  write_reconfiguration_direct_register_mode_1(i,0x0002,tx_reconfig_data[1]); // Line 6 (offset 2)
//		  write_reconfiguration_direct_register_mode_1(i,0x0004,tx_reconfig_data[2]); // Line 8 (offset 4)
//		  }	
		  
		  //for (i = (2*NUMBER_OF_LINKS); i < (4+(2*NUMBER_OF_LINKS)) ; i++) // This will reconfigure the PLL 8 .. 11
		  //{
		  if (DEBUG == 1)
				printf("Reconfigure logical channel with new PLL info %d\n",i); //This will reconfigure the PLL 8 
		  write_reconfiguration_direct_register_mode_1(8,0x000C,tx_reconfig_data[3]); // Line 4 (offset C)
		  write_reconfiguration_direct_register_mode_1(8,0x000E,tx_reconfig_data[4]); // Line 6 (offset E)
		  write_reconfiguration_direct_register_mode_1(8,0x000F,tx_reconfig_data[5]); // Line 7 (offset F)		  
		  //}			
		
		  write_reconfiguration_refclk(8, Refclk_select_tx); // This will reconfigure the PLL with the right refclk

		  
//		  for (i = (2*NUMBER_OF_LINKS); i < (4+(2*NUMBER_OF_LINKS)) ; i++)
//		  {
//		  write_reconfiguration_refclk(i, Refclk_select_tx); // This will reconfigure the PLL with the right refclk
//		  }			  
	
	  
		Control_Reg[0] = Control_Reg[0] | (0x8000);
		IOWR_ALTERA_AVALON_PIO_DATA(CONTROL_REG_BASE,Control_Reg[0]);
		usleep(10);
		
		Control_Reg[0] = Control_Reg[0] & (0x7FFF);
		IOWR_ALTERA_AVALON_PIO_DATA(CONTROL_REG_BASE,Control_Reg[0]);
	  
		usleep(1000000);		  
		  
		printf("\nChanged Datarate on Tx side");	
		
		show_coreclks = 0; // don't show the coreclk's yet as it takes time to measure them.  
		
      break;
		
	case 'r':
	case 'R' : //Change Datarate on Rx using Software
		
	   printf("\nCurrent selected datarate on Rx is ");
			  switch (datarate_rx)
			  {
			  case  RATE_5400MBPS    : printf("5400 Mbps from 135 Mhz Reference Clock\n"); break;
			  case  RATE_2700MBPS    : printf("2700 Mbps from 135 Mhz Reference Clock\n"); break;
			  case  RATE_1620MBPS    : printf("1620 Mbps from 135 Mhz Reference Clock\n"); break;
			  case  RATE_2970MBPS    : printf("2970 Mbps from 297 Mhz Reference Clock\n"); break;				  
			  default : break;
			  }	

       do {
        printf("\n");  
        
        printf("    New Rx Datarate             \n");
        printf("    ============================                \n");
        printf("    For 5400 Mbps from 135 Mhz Reference Clock choose '0' \n");
        printf("    For 2700 Mbps from 135 Mhz Reference Clock choose '1' \n");
        printf("    For 1620 Mbps from 135 Mhz Reference Clock choose '2' \n");
        printf("    For 2970 Mbps from 297 Mhz Reference Clock choose '3' \n");			 
        printf("    No Change    choose 'X' \n\n");    
        printf("    Choice :");             
        
        temp = 0xffffffff;
                                             
        rx_char   = input_char();
        switch (rx_char)
        {
        case '0': temp =  RATE_5400MBPS; Refclk_select_rx = RX_5400MBPS_REFCLK; 
						rx_reconfig_data[0] = RX_5400MBPS_4 ;

						break;			  
        case '1': temp =  RATE_2700MBPS; Refclk_select_rx = RX_2700MBPS_REFCLK;
						rx_reconfig_data[0] = RX_2700MBPS_4 ;

						break;				  
        case '2': temp =  RATE_1620MBPS; Refclk_select_rx = RX_1620MBPS_REFCLK;
						rx_reconfig_data[0] = RX_1620MBPS_4 ;

						break;				  
        case '3': temp =  RATE_2970MBPS; Refclk_select_rx = RX_2970MBPS_REFCLK; 
						rx_reconfig_data[0] = RX_2970MBPS_4 ;

						break;				  
        case 'x': 
        case 'X': temp =  datarate_rx;break;
        default : break;
        }
        } while (temp == 0xffffffff);
 
        datarate_rx = temp; 			  
		
		
		// Set Datarate 
		Control_Reg[0] = (Control_Reg[0] & (0xF9FF)) | (datarate_rx << 9);
		  
		IOWR_ALTERA_AVALON_PIO_DATA(CONTROL_REG_BASE,Control_Reg[0]);
		
		  for (i = 0; i < (NUMBER_OF_LINKS) ; i++) // This will reconfigure the Rx channels 0..3
		  {
		  if (DEBUG == 1)
				printf("Reconfigure logical channel with new Rx channel info %d\n",i);
		  write_reconfiguration_direct_register_mode_1(i,0x000C,rx_reconfig_data[0]);  // Line 4 (offset C) 
		  //write_reconfiguration_direct_register_mode_1(i,0x0019,rx_reconfig_data[1]);  // Line 13 (offset 0x19) 
		  //write_reconfiguration_direct_register_mode_1(i,0x001A,rx_reconfig_data[2]);  // Line 14 (offset 0x1A) 
		  }			  
	
		  for (i = 0; i < (NUMBER_OF_LINKS) ; i++)
		  {
		  write_reconfiguration_refclk(i, Refclk_select_rx); // This will reconfigure the Rx channels with the correct referenceclock
		  }			  
	
	  
		Control_Reg[0] = Control_Reg[0] | (0x8000);
		IOWR_ALTERA_AVALON_PIO_DATA(CONTROL_REG_BASE,Control_Reg[0]);
		usleep(10);
		
		Control_Reg[0] = Control_Reg[0] & (0x7FFF);
		IOWR_ALTERA_AVALON_PIO_DATA(CONTROL_REG_BASE,Control_Reg[0]);
	  
		usleep(1000000);		  
		  
		printf("\nChanged Datarate on Rx side");	
		
		show_coreclks = 0; // don't show the coreclk's yet as it takes time to measure them.  
		
      break;
		
		case 'm':
		case 'M':

		eye1d = 1;
		if (datarate_rx == RATE_1620MBPS)
			eyeqbw = 3; // Note that handbook mentions to use 1 but this is effectively the worst, 3 is best. Datarate > 1.6 and <= 2.5 Gbps
		else	
			eyeqbw = 2; //Datarate > 2.5 Gbps and <= 7.5 Gbps
		
		eyeq_status = read_reconfiguration_EyeQ(SelectedChannel,0x3);
		eyeq_control = eyeq_status;
		eyeq_control = (eyeq_control & (0x0000DFFF)) | (eye1d << 13);
		eyeq_control = (eyeq_control & (0x0000F9FF)) | (eyeqbw << 9);
		
		write_reconfiguration_EyeQ(SelectedChannel,0x3,eyeq_control); // Set eye1d and Eyeq Bandwidth
		
		write_reconfiguration_EyeQ(SelectedChannel,0x0,0x0001); // Enable Eye Monitor
		
	    ///////////////////////////////////////////////////////////////////////				
		// Go through all phase steps one by one and measure the BER
		///////////////////////////////////////////////////////////////////////	
            
		for (i = 0; i < 32 ; i++)
		//for (i = 0; i < 64 ; i++)
			{ 

			write_reconfiguration_EyeQ(SelectedChannel,0x1,i); // Select horizontal phase
							
			usleep(100000); // wait for Prbslogic to stabilize
			
								   
			// Reset ErrorCount on Selected Channel

	     Control_Reg[0] = Control_Reg[0] | (0x2000);
        IOWR_ALTERA_AVALON_PIO_DATA(CONTROL_REG_BASE,Control_Reg[0]);
        
        Control_Reg[0] = Control_Reg[0] & (0xDFFF);
        IOWR_ALTERA_AVALON_PIO_DATA(CONTROL_REG_BASE,Control_Reg[0]);	
		
			TimeInterval = BERInterval * 1000;
	
			// Wait for programmed time interval
			do
			{
			Counter_1ms_Reg[0]     = IORD_ALTERA_AVALON_PIO_DATA(COUNTER_1MS_REG_BASE);
			} while (Counter_1ms_Reg[0] < TimeInterval);                  
             
    
        
            //Check if PRBS Lock is valid
            
			Channel_Reg[SelectedChannel]     = Read_Channel_Reg(SelectedChannel);
			
			Locked[SelectedChannel]  		= 0x0001 & (Channel_Reg[SelectedChannel] >> 15);  
	
            //Read ErrorCount registers
            
            if (Locked[SelectedChannel] == 1)
            {			  
				Counter_1ms_Reg[0]     = IORD_ALTERA_AVALON_PIO_DATA(COUNTER_1MS_REG_BASE);
				ErrorCount_Reg_L[SelectedChannel]     = Read_ErrorCount_L_Reg(SelectedChannel);
				ErrorCount_Reg_H[SelectedChannel]     = Read_ErrorCount_H_Reg(SelectedChannel);
				
						 
				ErrorCount_Array[i] = ((float) (ErrorCount_Reg_H[SelectedChannel]) *  MULTIPLIER * MULTIPLIER) + (float) (ErrorCount_Reg_L[SelectedChannel]);
	
	  
			   Totalbits_tx[SelectedChannel] = (float) (Counter_1ms_Reg[0]) * (float) (Bitrate[datarate_tx]*1000);
					
				if (ErrorCount_Array[i] > 0)
					{
				  BER_Array[i] = ((ErrorCount_Array[i])) / Totalbits_tx[SelectedChannel];
				  //printf("\nErrorCount %6d %6d",ErrorCount_Reg_H[SelectedChannel],ErrorCount_Reg_L[SelectedChannel]);
				  //printf("ErrorCount : %5e\n", ErrorCount_Array[i]);
				  }
				else 
				  BER_Array[i] = 0;  
				  
				if (ErrorCount_Array[i] > 0)
						printf("Ch %1d Phase Step : %2d BER Estimate : %4e    ErrorCount  : %4e \n",SelectedChannel,i,(BER_Array[i]),ErrorCount_Array[i]);
					else
						printf("Ch %1d Phase Step : %2d BER          : 0\n",SelectedChannel,i);			    
            
            }
            else
			{
				BER_Array[i] = 1;
				printf("Ch %1d Phase Step : %2d BER          : 1 (no Prbs Lock)\n",SelectedChannel,i);	
			}
			
                 ///////////////////////////////////////////////////////////////////////				
				// Generate a table for the BER
                ///////////////////////////////////////////////////////////////////////				

				if ( (BER_Array[i] == 1) )
					Table[i][0] = 1;
				else
					Table[i][0] = 0;

				if ( (BER_Array[i] < 1) && (BER_Array[i] > 0.1) )
					Table[i][1] = 1;
				else
					Table[i][1] = 0;

				if ( (BER_Array[i] < 0.1) && (BER_Array[i] > 0.01) )
					Table[i][2] = 1;
				else
					Table[i][2] = 0;
					
				if ( (BER_Array[i] < 0.01) && (BER_Array[i] > 0.001) )
					Table[i][3] = 1;
				else
					Table[i][3] = 0;

				if ( (BER_Array[i] < 0.001) && (BER_Array[i] > 0.0001) )
					Table[i][4] = 1;
				else
					Table[i][4] = 0;

				if ( (BER_Array[i] < 0.0001) && (BER_Array[i] > 0.00001) )
					Table[i][5] = 1;
				else
					Table[i][5] = 0;
					
				if ( (BER_Array[i] < 0.00001) && (BER_Array[i] > 0.000001) )
					Table[i][6] = 1;
				else
					Table[i][6] = 0;																									

				if ( (BER_Array[i] < 0.000001) && (BER_Array[i] > 0.0000001) )
					Table[i][7] = 1;
				else
					Table[i][7] = 0;	

				if ( (BER_Array[i] < 0.0000001) && (BER_Array[i] > 0.00000001) )
					Table[i][8] = 1;
				else
					Table[i][8] = 0;	

				if ( (BER_Array[i] < 0.00000001) && (BER_Array[i] > 0.000000001) )
					Table[i][9] = 1;
				else
					Table[i][9] = 0;

				if ( (BER_Array[i] < 0.000000001) && (BER_Array[i] > 0.0000000001) )
					Table[i][10] = 1;
				else
					Table[i][10] = 0;

				if ( (BER_Array[i] < 0.0000000001) && (BER_Array[i] > 0.00000000001) )
					Table[i][11] = 1;
				else
					Table[i][11] = 0;

				if ( (BER_Array[i] < 0.00000000001) )
					Table[i][12] = 1;
				else
					Table[i][12] = 0;
																																				
		} // for loop
		
		//printf("\n BATHTUB CURVE ESTIMATE ACROSS 64 PHASES\n");
	    printf("\n BATHTUB CURVE ESTIMATE ACROSS 32 PHASES (one UI)\n");
       
	       ///////////////////////////////////////////////////////////////////////				
		   // Determine eye opening
           ///////////////////////////////////////////////////////////////////////				
	
		eye_phase = 0;
		BER_Minimum = 1;
		BER_Minimum_phase = 0;
		 
        for (i = 0; i <32 ;i++)
        {
			if (BER_Array[i] == 0)
               eye_phase = eye_phase + 1;
            if (BER_Array[i] < BER_Minimum )
				{
				BER_Minimum = BER_Array[i];
				BER_Minimum_phase = i;
				}
        }
        
			

         eye = (float) eye_phase/32;
        
        start_value = 0;
        end_value = 0;
        
        for (i = 1; i <32 ;i++)
        {
          if (BER_Array[i] != BER_Array[i-1])
          {
               if (BER_Array[i] == 0)
					start_value = i;
               else if (BER_Array[i-1] == 0)
                     end_value = i-1;
           }
        }
        //printf("\nstart_value : %2d",start_value);
        //printf("\nend_value   : %2d",end_value);
        if (start_value > end_value )
			end_value = end_value + 32;
		

		//printf("\nend_value   : %2d",end_value);	
		if (eye_phase == 0)
			optimum_phase = BER_Minimum_phase ;
		else	
			optimum_phase = start_value + eye_phase/2;
			
		//printf("\noptimum phase   : %2d",optimum_phase );
		if (optimum_phase >= 32 )
			{
			table_phase_start = optimum_phase - 15;
			optimum_phase = optimum_phase -32;
			}
		else
			{
			table_phase_start = optimum_phase -15;
			if (table_phase_start < 0)
				table_phase_start = table_phase_start + 32;
			}
		//printf("\noptimum phase   : %2d",optimum_phase );
		//printf("\ntable_phase_start   : %2d",table_phase_start );	
      
	   ///////////////////////////////////////////////////////////////////////				
	   // Print Centered Eye
	   ///////////////////////////////////////////////////////////////////////				
       printf("\nPhase   ");
       for (i =0;i <32; i++)
			{
			index = table_phase_start + i;
			if (index > 31)
				index = index -32;
			if (index < 10) 
				printf("  ");
			else if ((index >= 10) && (index < 20))
			    printf(" 1");
			else if ((index >= 20) && (index < 30))
				printf(" 2");
			else
				printf(" 3");
			}
	  printf("\nStep    ");
       for (i =0;i <32; i++)
			{
			index = table_phase_start + i;
			if (index > 31)
				index = index -32;
			if (index < 10) 
				printf("%2d",index);
			else if ((index >= 10) && (index < 20))
			    printf("%2d",index-10);
			else if ((index >= 20) && (index < 30))
				printf("%2d",index-20);
			else
				printf("%2d",index-30);
			}			
	   printf("\n========================================================================\n");	   	   
	   for (j = 0; j < 13 ; j++)
			{   
			printf("\n10E-%2d :",j);  
				for (i = 0; i < 32 ; i++)
				{ 
					index = table_phase_start + i;
					if (index > 31)
						index = index -32;				    
					if (Table[index][j] == 1)
						printf(" *");
					else
						printf("  ");
				}
			}
		
		printf("\n");				
                
         printf("\n'Eye' Opening Ch %1d: %2d phases",SelectedChannel,eye_phase);
         printf("\n'Eye' Opening Ch %1d: %.2f UI",SelectedChannel,eye);
         printf("\nOptimum Phase Ch %1d: %2d",SelectedChannel,optimum_phase);
			
   write_reconfiguration_EyeQ(SelectedChannel,0x0,0x0000); // Disable Eye Monitor			
		
		
		break;
			

	case 'l':
	case 'L':

			for (j =0; j < 100; j++)
			{
		  for (i = 0; i < NUMBER_OF_LINKS; i++)
		  { 
			  
  write_reconfiguration(i+4,VOD,tx_vodctrl[i]); //Tx only
  write_reconfiguration(i+4,PRE_TAP,tx_preemp_0t[i]); //Tx only
  write_reconfiguration(i+4,POST_TAP1,tx_preemp_1t[i]);	//Tx only  
  write_reconfiguration(i+4,POST_TAP2,tx_preemp_2t[i]);	//Tx only
  write_reconfiguration(i,EQ_GAIN,rx_eqdcgain[i]);	 
  write_reconfiguration(i,EQ_CTRL,rx_eqctrl[i]);	    
		  } 				 
		  }
		  printf("\n Performed PMA configuration on all channels 100 times\n");
  break;
		  
     case '0':
        printf("\nTerminating Program.\n");
        return 0;
       break;
  
      default:
        printf("\n\tPlease enter a value from 0 to T\n\n\n");
        break;
    }
 }
 return 0;
}
